#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using System.Text;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.Design;
#endregion

namespace NinjaTrader.Indicator
{
    /// <summary> 	//Modification of Scalper indicator with triggers added for use in automated strategys (Contact:DerekPhelps22@gmail.com)
	/// 			//Short ScalpBar is highest high with 2 subsequest lower lows (reverse for long Scalpbars)
	/// 			//Note: ScalpBars are not identified in real (current bar) time but by definition cannot be marked as a scalp bar until several (sometimes many) bars go by until requirements are met
    /// Adaptation of Scalper strategy (Gumphrie) with signaling added for automated strategys and a few new features added.  DerekPhelps22@gmail.com
	/// AutoStrategy Usage: 	ScalperH().Direction : Direction signal of last pivot : 1=Low pivot (Scalp Long), -1=HighPivot (Scalp Short) 
	/// 						ScalperH().TriggerBar: BarNumber of last pivot.
	/// 						ScalperH().TriggerHigh: High of last pivot bar.
	/// 						ScalperH().TriggerLow: Low of last trigger bar.
	/// 						ScalperH().EarlyTrigger: Direction signal of possible trigger (only first of 2 higher highs or lower lows identified) Green or Red triangle:=1ScalpLong,-1ScalpShort
    /// </summary>
    [Description("Adaptation of Scalper strategy with signaling added for automated strategys and a few new features.  DerekPhelps22@gmail.com")]
    public class ScalperH : Indicator
    {
        #region Variables
            private SolidBrush ScalpBarBrush = new SolidBrush(Color.Navy);
		
			private int 	counter 		= 0;
			private int 	ExtremeBar 		= 0;
		    private int 	lastBar			= 0;

			private bool 	HighPainted 	= false;
//			private bool 	LowPainted = true;
			private bool 	showTriggers 	= true;
			private double 	ExtremeBarHigh 	= 0;
            private double 	TriggerBarLow 	= 0;
			private double 	ExtremeBarLow 	= 0;
            private double 	TriggerBarHigh 	= 0;
			private int 	TriggerBarLowNo	= 0;
			private int 	TriggerBarHighNo= 0;
			private bool	alert1			= false;	//generate an alert when a new scalp bar is identified.
			private int		width			= 2;
		
			private DataSeries  scalpSeries;
			private DataSeries 	open;
			private DataSeries 	high;
			private DataSeries 	low;
			private DataSeries 	close;
		
			private  int	direction 		= 0;
			private  double	triggerHigh 	= 0;
			private  double	triggerLow 		= 0;
			private  int	triggerBar		= 0;
			private  int	earlyTrigger 	= 0;
		
        #endregion
//
		protected override void Initialize()
        {
        	Overlay = true;     
			scalpSeries = new DataSeries(this);
			open  = new DataSeries(this);
			high  = new DataSeries(this);
			low   = new DataSeries(this);
			close = new DataSeries(this);			
        	CalculateOnBarClose = true;
        }
//
        protected override void OnBarUpdate()
        {	open.Set(Open[0]);
			high.Set(High[0]);
			low.Set(Low[0]);
			close.Set(Close[0]);
            if (CurrentBar <= 4)	{	scalpSeries.Set(0);	}
            else
            {	//Up 
				if (HighPainted==false)
				{	if ((High[1] < High[2]) && (ExtremeBar==0))
					//if ((High[1] < High[2]) && (2 == Bars.HighestBar(3)) && (0==ExtremeBar)) 
					{
                		ExtremeBar		= 2; 
                		ExtremeBarHigh	= High[2]; 
                		TriggerBarLow	= Low[1]; 
						TriggerBarLowNo	= CurrentBar-1;
						DrawTriangleDown("MaybePivotDown",true,ExtremeBar,ExtremeBarHigh+TickSize,Color.Green);	//Temp tag potential scalp bar candidates until requirements met
						RemoveDrawObject("MaybePivotUp");
						earlyTrigger	= -1;
						
					//	if (ExtremeBarLow > High[4]) ExtremeBar = 4;
                    }
						
            		if (High[0] > ExtremeBarHigh)
					{	RemoveDrawObject("MaybePivotDown");
						earlyTrigger	= 0;
                		ExtremeBar 		= 0; 
                		ExtremeBarHigh 	= 0; 
                		TriggerBarLow 	= 0; 
						TriggerBarLowNo = 0;
						counter			=-1; 
					}
            		if (ExtremeBar!=0) counter++; 
            		if ((Close[0] < TriggerBarLow) && (ExtremeBar!=0))
					{
                 		ExtremeBar = ExtremeBar + counter;
                 		HighPainted = true; 	
                        scalpSeries.Set(CurrentBar-ExtremeBar);	
						
						direction=-1;						//	High Pivot, Scalp direction is down
						triggerBar=CurrentBar-ExtremeBar;	//	Bar count of the pivot bar
						triggerLow=Low[ExtremeBar];			//	Low of the Pivot bar
						triggerHigh=High[ExtremeBar];		//	High of the Pivot bar
						DrawArrowLine("AD"+CurrentBar.ToString(),true,ExtremeBar,High[ExtremeBar]+TickSize,ExtremeBar,Low[ExtremeBar]-TickSize,ScalpBarBrush.Color,DashStyle.Solid,Width);
						if (showTriggers)	DrawTriangleDown("TD"+CurrentBar.ToString(),false,0,(High[0]-Low[0])/2+Low[0],Color.Yellow);	 //Tag current bar to show the lag that scalp bar was detected on
						if (Alert1) 		Alert("Alert1",Priority.Low,"HighPivot:Scalp Short!","AutoChase.wav",1000,Color.Black,Color.Red);
					
						ExtremeBar=0; counter=-1;			//	Reset for next pivot

                    }
                }
				//Down
				 if (HighPainted==true)
				{
					if ((Low[1] > Low[2]) &&  (ExtremeBar==0))
					//if ((Low[1] > Low[2]) && (2 == Bars.LowestBar(3)) && (0==ExtremeBar)) 
					{
                  		ExtremeBar 			= 2; 
                  		ExtremeBarLow 		= Low[2]; 
                  		TriggerBarHigh		= High[1]; 
						TriggerBarHighNo 	= CurrentBar-1;
						DrawTriangleUp("MaybePivotUp",true,ExtremeBar,ExtremeBarLow-TickSize,Color.Red);
						RemoveDrawObject("MaybePivotDown");
						earlyTrigger		= 1;

                  	//	if (ExtremeBarLow > Low[4]) ExtremeBar = 4;
            		}
            		if (Low[0] < ExtremeBarLow)
					{ 	RemoveDrawObject("MaybePivotUp");
						earlyTrigger 		= 0;
                  		ExtremeBar 			= 0; 
                  		ExtremeBarLow 		= 0; 
                  		TriggerBarHigh		= 0; 
						TriggerBarHighNo	= 0; 
                  		counter				=-1; 
            		}
					if (ExtremeBar!=0) counter++; 
					if ((Close[0] > TriggerBarHigh) && (ExtremeBar!=0))
					{
                  		ExtremeBar = ExtremeBar + counter;                 
						HighPainted = false;
                    	scalpSeries.Set(CurrentBar-ExtremeBar);	
						direction= 1;						//	Low Pivot, Scalp direction is Up
						triggerBar=CurrentBar-ExtremeBar;	//	Bar count of the pivot bar
						triggerLow=Low[ExtremeBar];			//  Low of the Pivot bar
						triggerHigh=High[ExtremeBar];		//	High of the Pivot bar
						DrawArrowLine("AU"+CurrentBar.ToString(),true,ExtremeBar,Low[ExtremeBar]+TickSize,ExtremeBar,High[ExtremeBar]-TickSize,ScalpBarBrush.Color,DashStyle.Solid,Width); //
						if (showTriggers) 	DrawTriangleUp("TU"+CurrentBar.ToString(),true,0,(High[0]-Low[0])/2+Low[0],Color.Yellow);	//Tag current bar to show the lag that scalp bar was detected on
						if (Alert1) 		Alert("Alert1",Priority.Low,"LowPivot:Scalp Long!","AutoChase.wav",1000,Color.Black,Color.Green);

						ExtremeBar=0; counter=-1;			//	Reset for next pivot
            		}
				}
            }
			lastBar=CurrentBar;
        }
//

        #region Properties            
  		/// <summary>
		/// Return Scalp Direction
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public int Direction
		{	get { Update(); return direction; }		}
//
		[Browsable(false)]
		[XmlIgnore()]
		public int EarlyTrigger
		{	get { Update(); return earlyTrigger; }		}
//
		[Browsable(false)]
		[XmlIgnore()]
		public double TriggerHigh
		{	get { Update(); return triggerHigh; }		}
//
		[Browsable(false)]
		[XmlIgnore()]
		public double TriggerLow
		{	get { Update(); return triggerLow; }		}
//
		[Browsable(false)]
		[XmlIgnore()]
		public int TriggerBar
		{	get { Update(); return triggerBar; }		}
//
		[Browsable(false)]
    	public string ScalpBarColorSerialize
    	{  	get { return SerializableColor.ToString(this.ScalpBarColor); }
        	set { this.ScalpBarColor = SerializableColor.FromString(value); }
    	}
		
		[Description("Default Colour for Scalp Bar"), XmlIgnore, VisualizationOnly]
		[Category("Scalp Bar")]
		[NinjaTrader.Gui.Design.DisplayName("Scalp bar color")]
		public Color ScalpBarColor
        {
			get { return this.ScalpBarBrush.Color; }
        	set { this.ScalpBarBrush = new SolidBrush(value); }
        }
		
		[Description("Scalp bars are not created and drawn until *ScalpHigh: High followed by lower high and 2 lower lows, Reverse for ScalpLows")]
        [Category("Scalp Bar")]
		[NinjaTrader.Gui.Design.DisplayName("Highlight current trigger bars")]
		public bool ScalperTriggers
        {
            get { return showTriggers; }
			set { showTriggers = value; }
        }

		[Description("Create an alert when a scalp bar is identified")]
        [Category("Scalp Bar")]
		[NinjaTrader.Gui.Design.DisplayName("Scalp Bar Alert")]
		public bool Alert1
        {
            get { return alert1; }
			set { alert1 = value; }
        }
		
		[Description("Arrow Width")]
        [Category("Visual")]
		[NinjaTrader.Gui.Design.DisplayName("Width")]
		public int Width
        {
            get { return width; }
			set { width = value; }
        }

        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private ScalperH[] cacheScalperH = null;

        private static ScalperH checkScalperH = new ScalperH();

        /// <summary>
        /// Adaptation of Scalper strategy with signaling added for automated strategys and a few new features.  DerekPhelps22@gmail.com
        /// </summary>
        /// <returns></returns>
        public ScalperH ScalperH()
        {
            return ScalperH(Input);
        }

        /// <summary>
        /// Adaptation of Scalper strategy with signaling added for automated strategys and a few new features.  DerekPhelps22@gmail.com
        /// </summary>
        /// <returns></returns>
        public ScalperH ScalperH(Data.IDataSeries input)
        {
            if (cacheScalperH != null)
                for (int idx = 0; idx < cacheScalperH.Length; idx++)
                    if (cacheScalperH[idx].EqualsInput(input))
                        return cacheScalperH[idx];

            lock (checkScalperH)
            {
                if (cacheScalperH != null)
                    for (int idx = 0; idx < cacheScalperH.Length; idx++)
                        if (cacheScalperH[idx].EqualsInput(input))
                            return cacheScalperH[idx];

                ScalperH indicator = new ScalperH();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                Indicators.Add(indicator);
                indicator.SetUp();

                ScalperH[] tmp = new ScalperH[cacheScalperH == null ? 1 : cacheScalperH.Length + 1];
                if (cacheScalperH != null)
                    cacheScalperH.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheScalperH = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Adaptation of Scalper strategy with signaling added for automated strategys and a few new features.  DerekPhelps22@gmail.com
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.ScalperH ScalperH()
        {
            return _indicator.ScalperH(Input);
        }

        /// <summary>
        /// Adaptation of Scalper strategy with signaling added for automated strategys and a few new features.  DerekPhelps22@gmail.com
        /// </summary>
        /// <returns></returns>
        public Indicator.ScalperH ScalperH(Data.IDataSeries input)
        {
            return _indicator.ScalperH(input);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Adaptation of Scalper strategy with signaling added for automated strategys and a few new features.  DerekPhelps22@gmail.com
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.ScalperH ScalperH()
        {
            return _indicator.ScalperH(Input);
        }

        /// <summary>
        /// Adaptation of Scalper strategy with signaling added for automated strategys and a few new features.  DerekPhelps22@gmail.com
        /// </summary>
        /// <returns></returns>
        public Indicator.ScalperH ScalperH(Data.IDataSeries input)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.ScalperH(input);
        }
    }
}
#endregion
