
#region Using declarations
using System;
using System.ComponentModel;
using System.Drawing;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion
namespace NinjaTrader.Indicator
{	/// Created by DerekPhelps22@gmail.com for Shaun Overton (OneStepRemoved.com)  09/25/13
	/// Prototype: Slope(Indicates indicatorName,int Period, int Smooth, int Lookback, int TriggerAngle, View viewname )
	/// Example  : Add(Slopes(Indicator.Slopes.Indicates.SMA,14,3,30,Indicator.Slopes.Views.Indicator));
	/// Methods	 : 	Trigger : Slopes(  Indicator.Slopes.Indicates.SMA,2,14,3,30,Indicator.Slopes.Views.Indicator).Trigger[0]; 	//Returns the direction of slope: 1=Up,-1=Down,0=Undefined(less then TriggerAngle)
	///				Angle	: Slopes(  Indicator.Slopes.Indicates.SMA,2,14,3,30,Indicator.Slopes.Views.Indicator).Angle[0]; 	//Returns the angle in degrees of slope
	/// 			Value	: Slopes(  Indicator.Slopes.Indicates.SMA,2,14,3,30,Indicator.Slopes.Views.Indicator).Value[0]; 	//Returns the value of the selected indicator
	/// </summary>
	[Description("Determine slope of various selected indicators and trigger entries/exits on slope angle thresholds")]
	public class Slopes : Indicator
	{
		#region Variables
			// User variables for Choosing which indicator to apply slope to
			public Indicates	indicate	= Indicates.SMA;			// Dropdown Control for selecting secondary indicator
				public enum Indicates			{	SMA = 1,		EMA	= 2,	HMA=3,	LinReg=4,		RSI	= 5,	}
			// User dropdown to determine what to plot on screen
			public Views		view		= Views.Indicator;			// Dropdown Control for selecting secondary indicator
				public enum Views				{	Indicator = 1,		Angle	= 2,	Trigger=3,			}
					
			private int		period			= 14;		//Lookback Period for Selected Indicator
			private int		smooth			= 10;		//Only valid for RSI indicator
			private int 	lookback 		= 1; 
			private double 	triggerAngle 	= 30;
			private Color	valueColor;	//	= Color.Lime;			//Default colors for Plots
			private Color	angleColor;	//	= Color.Transparent;	//Default colors for Plots
			private Color	triggerColor;//	= Color.Transparent;	//Default colors for Plots
		#endregion

		protected override void Initialize()
		{	Overlay = true;
			Add(new Plot(new Pen(valueColor,	2),	PlotStyle.Line, "Value"));
			Add(new Plot(new Pen(angleColor, 	2), PlotStyle.Line, "Angle"));
			Add(new Plot(new Pen(triggerColor, 	2), PlotStyle.Line, "Trigger"));
		}
		//ENDInitialize
		protected override void OnStartUp()
		{	switch(view)	//
			{	case Views.Indicator:			{	Plots[0].Pen=new Pen(Color.Lime);		Plots[1].Pen = new Pen(Color.Transparent);	Plots[2].Pen = new Pen(Color.Transparent);	}	break;
				case Views.Trigger:				{	Plots[0].Pen=new Pen(Color.Transparent);Plots[1].Pen = new Pen(Color.Transparent);	Plots[2].Pen = new Pen(Color.Lime);			}	break;
				case Views.Angle:				{	Plots[0].Pen=new Pen(Color.Transparent);Plots[1].Pen = new Pen(Color.Lime);			Plots[2].Pen = new Pen(Color.Transparent);	}	break;
			}	if (view!=Views.Indicator||indicate==Indicates.RSI) ScaleJustification=ScaleJustification.Left; else ScaleJustification=ScaleJustification.Right;
		}//ENDOnStartup
		//
		protected override void OnBarUpdate()
		{	//Apply selected indicator as input for Slope calculations
			if (CurrentBar<period) return;
			switch(indicate)	//
			{	case Indicates.SMA:				{	Value.Set(SMA(SMA(period),smooth)[0]);		}	break;
				case Indicates.EMA:				{	Value.Set(SMA(EMA(period),smooth)[0]);		}	break;
				case Indicates.HMA:				{	Value.Set(SMA(HMA(period),smooth)[0]);		}	break;
				case Indicates.LinReg: 			{	Value.Set(SMA(LinReg(period),smooth)[0]);	}	break;
				case Indicates.RSI: 			{	Value.Set(RSI(period,smooth).Avg[0]);		}	break;
			}
			if(CurrentBar<BarsRequired) return;
			Angle.Set((180/Math.PI) *(Math.Atan((Slope(Value,lookback,0)/ATR(Value,period)[0]))));
			//Print(Time[0].ToString("G")+"  "+(180/Math.PI) *(Math.Atan((Slope(Value,lookback,0)/ATR(Value,period)[0]))));
			
			//Apply colors and set trigger dependent on angle
			if (Angle[0] >= triggerAngle) 	{	PlotColors[0][0]=Color.Green; 	Trigger.Set(1);			}
			else 
			if (Angle[0] <= -triggerAngle) 	{	PlotColors[0][0]=Color.Red; 	Trigger.Set(-1);		}
			else 
			{	PlotColors[0][0]=Color.Yellow; 	Trigger.Set(0);			}
		}//ENDOnBarUpdate()
		
		#region Properties
		//
		[Browsable(false)]        [XmlIgnore()]
        public DataSeries Value				{	get { return Values[0];				}		}
		//
		[Browsable(false)]        [XmlIgnore()]
        public DataSeries Angle				{	get { return Values[1];				}		}
		//
		[Browsable(false)]        [XmlIgnore()]
        public DataSeries Trigger			{	get { return Values[2];				}		}
		//		
		[Description("Select what to plot on chart")]
		[GridCategory("Parameters")]
		public NinjaTrader.Indicator.Slopes.Views 	View
		{   get { return view; }				set { view = value;					}		}
 		//
		[Description("Indicator to apply to Slope calculations")]
		[GridCategory("Parameters")]
		public NinjaTrader.Indicator.Slopes.Indicates 	Indicate
		{   get { return indicate; }			set { indicate = value;				}		}
 		//
		[Description("Numbers of bars used for calculations")]
		[GridCategory("Parameters")]
		public int Period
		{	get { return period; }				set { period = Math.Max(1, value);	}		}
		//
		[Description("Smoothing Period of Slope angle (only valid for RSI selection")]
		[Category("Parameters")]
		public int SmoothSlope
		{	get { return smooth; }				set { smooth = Math.Max(1, value);	}		}
		//
		[Description("Numbers of bars used in calculating slope angle")]
		[Category("Parameters")]
		public int Lookback
		{	get { return lookback; }			set { lookback=Math.Max(1, value);	}		}
		//
		[Description("Desired angle of Slope Trigger")]
		[Category("Parameters")]
		public double TriggerSlope
		{	get { return triggerAngle; }		set { triggerAngle=Math.Min(89,value);}		}
		#endregion
	
	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private Slopes[] cacheSlopes = null;

        private static Slopes checkSlopes = new Slopes();

        /// <summary>
        /// Determine slope of various selected indicators and trigger entries/exits on slope angle thresholds
        /// </summary>
        /// <returns></returns>
        public Slopes Slopes(NinjaTrader.Indicator.Slopes.Indicates indicate, int lookback, int period, int smoothSlope, double triggerSlope, NinjaTrader.Indicator.Slopes.Views view)
        {
            return Slopes(Input, indicate, lookback, period, smoothSlope, triggerSlope, view);
        }

        /// <summary>
        /// Determine slope of various selected indicators and trigger entries/exits on slope angle thresholds
        /// </summary>
        /// <returns></returns>
        public Slopes Slopes(Data.IDataSeries input, NinjaTrader.Indicator.Slopes.Indicates indicate, int lookback, int period, int smoothSlope, double triggerSlope, NinjaTrader.Indicator.Slopes.Views view)
        {
            if (cacheSlopes != null)
                for (int idx = 0; idx < cacheSlopes.Length; idx++)
                    if (cacheSlopes[idx].Indicate == indicate && cacheSlopes[idx].Lookback == lookback && cacheSlopes[idx].Period == period && cacheSlopes[idx].SmoothSlope == smoothSlope && Math.Abs(cacheSlopes[idx].TriggerSlope - triggerSlope) <= double.Epsilon && cacheSlopes[idx].View == view && cacheSlopes[idx].EqualsInput(input))
                        return cacheSlopes[idx];

            lock (checkSlopes)
            {
                checkSlopes.Indicate = indicate;
                indicate = checkSlopes.Indicate;
                checkSlopes.Lookback = lookback;
                lookback = checkSlopes.Lookback;
                checkSlopes.Period = period;
                period = checkSlopes.Period;
                checkSlopes.SmoothSlope = smoothSlope;
                smoothSlope = checkSlopes.SmoothSlope;
                checkSlopes.TriggerSlope = triggerSlope;
                triggerSlope = checkSlopes.TriggerSlope;
                checkSlopes.View = view;
                view = checkSlopes.View;

                if (cacheSlopes != null)
                    for (int idx = 0; idx < cacheSlopes.Length; idx++)
                        if (cacheSlopes[idx].Indicate == indicate && cacheSlopes[idx].Lookback == lookback && cacheSlopes[idx].Period == period && cacheSlopes[idx].SmoothSlope == smoothSlope && Math.Abs(cacheSlopes[idx].TriggerSlope - triggerSlope) <= double.Epsilon && cacheSlopes[idx].View == view && cacheSlopes[idx].EqualsInput(input))
                            return cacheSlopes[idx];

                Slopes indicator = new Slopes();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.Indicate = indicate;
                indicator.Lookback = lookback;
                indicator.Period = period;
                indicator.SmoothSlope = smoothSlope;
                indicator.TriggerSlope = triggerSlope;
                indicator.View = view;
                Indicators.Add(indicator);
                indicator.SetUp();

                Slopes[] tmp = new Slopes[cacheSlopes == null ? 1 : cacheSlopes.Length + 1];
                if (cacheSlopes != null)
                    cacheSlopes.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheSlopes = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Determine slope of various selected indicators and trigger entries/exits on slope angle thresholds
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.Slopes Slopes(NinjaTrader.Indicator.Slopes.Indicates indicate, int lookback, int period, int smoothSlope, double triggerSlope, NinjaTrader.Indicator.Slopes.Views view)
        {
            return _indicator.Slopes(Input, indicate, lookback, period, smoothSlope, triggerSlope, view);
        }

        /// <summary>
        /// Determine slope of various selected indicators and trigger entries/exits on slope angle thresholds
        /// </summary>
        /// <returns></returns>
        public Indicator.Slopes Slopes(Data.IDataSeries input, NinjaTrader.Indicator.Slopes.Indicates indicate, int lookback, int period, int smoothSlope, double triggerSlope, NinjaTrader.Indicator.Slopes.Views view)
        {
            return _indicator.Slopes(input, indicate, lookback, period, smoothSlope, triggerSlope, view);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Determine slope of various selected indicators and trigger entries/exits on slope angle thresholds
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.Slopes Slopes(NinjaTrader.Indicator.Slopes.Indicates indicate, int lookback, int period, int smoothSlope, double triggerSlope, NinjaTrader.Indicator.Slopes.Views view)
        {
            return _indicator.Slopes(Input, indicate, lookback, period, smoothSlope, triggerSlope, view);
        }

        /// <summary>
        /// Determine slope of various selected indicators and trigger entries/exits on slope angle thresholds
        /// </summary>
        /// <returns></returns>
        public Indicator.Slopes Slopes(Data.IDataSeries input, NinjaTrader.Indicator.Slopes.Indicates indicate, int lookback, int period, int smoothSlope, double triggerSlope, NinjaTrader.Indicator.Slopes.Views view)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.Slopes(input, indicate, lookback, period, smoothSlope, triggerSlope, view);
        }
    }
}
#endregion
