#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

namespace NinjaTrader.Indicator
{
    /// <summary>
	///  This is a modification to the excelent TChannel2 indicator
	/// 
    /// V2: 4/1/2013 DerekPhelps22@gmail.com : Added public TradeTrigger property to make accessable from within an automated strategy
	/// Ex: TrendChannel(deviation,MAPeriod,RSIPeriod,LongTrigger,ShortTrigger).TradeTrigger==1 (Rising trend channel dot print) or -1 (Falling trend channel dot printed)
	/// where LongTrigger setting > 50 and ShortTrigger < 50
    /// </summary>
    [Description("Trend Channel of StandardDeviation of Movingaverage relation to RSI and ATR")]
    public class TrendChannel : Indicator
    {
        #region Variables
        // Wizard generated variables
            private int mALength = 21; // Default setting for MALength
            private double dev = 1.000; // Default setting for Const
			private double xl = 51;
			private double xs = 49;
			private int rSILength = 21;
        // User defined variables (add any user defined variables below)
			private int tradeTrigger = 0;
        #endregion
        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.Transparent), PlotStyle.Line, "MidLine"));
            Add(new Plot(Color.Red, PlotStyle.Dot, "UCrosses"));
            Add(new Plot(Color.Green, PlotStyle.Dot, "LCrosses"));
            Add(new Plot(Color.FromKnownColor(KnownColor.Red), PlotStyle.Line, "Upper"));
            Add(new Plot(Color.FromKnownColor(KnownColor.Green), PlotStyle.Line, "Lower"));
			
			Plots[1].Pen.Width = 1;
			Plots[2].Pen.Width = 1;
			Plots[3].Pen.DashStyle = DashStyle.Dot;
			Plots[4].Pen.DashStyle = DashStyle.Dot;
			
            CalculateOnBarClose	= true;
            Overlay				= true;
            PriceTypeSupported	= true;
        }

         protected override void OnBarUpdate()
        {	tradeTrigger=0;
            MidLine.Set(EMA(Input,MALength)[0]);
			double avg = SMA(ATR(1),MALength)[0];
            Upper.Set(MidLine[0] + avg*Dev);
            Lower.Set(MidLine[0] - avg*Dev);
			double ttt = RSI(Input,RSILength,1)[0];
			if (ttt > XL) 
			{	Values[2].Set(Lower[0]);
				tradeTrigger = 1;
			}	
			else
			if(ttt < XS) 
			{	Values[1].Set(Upper[0]);
				tradeTrigger = -1;
				
			}	
        }

        #region Properties
		/// TradeTrigger
		/// </Returns +1 or -1 depending if trend rising or falling
		[Browsable(false)]
		[XmlIgnore()]
		public int TradeTrigger
		{
			get { Update(); return tradeTrigger; }
		}
        [Browsable(false)]	
        [XmlIgnore()]		
        public DataSeries MidLine
        {
            get { return Values[0]; }
        }

        [Browsable(false)]	
        [XmlIgnore()]		
        public DataSeries Upper
        {
            get { return Values[3]; }
        }

        [Browsable(false)]	
        [XmlIgnore()]		
        public DataSeries Lower
        {
            get { return Values[4]; }
        }

        [Description("")]
        [Category("Parameters")]
        public int MALength
        {
            get { return mALength; }
            set { mALength = Math.Max(1, value); }
        }

        [Description("")]
        [Category("Parameters")]
        public int RSILength
        {
            get { return rSILength; }
            set { rSILength = Math.Max(1, value); }
        }

        [Description("")]
        [Category("Parameters")]
        public double Dev
        {
            get { return dev; }
            set { dev = Math.Max(0.00001, value); }
        }
        [Description("")]
        [Category("Parameters")]
        public double XL
        {
            get { return xl; }
            set { xl = Math.Max(0.00001, value); }
        }
        [Description("")]
        [Category("Parameters")]
        public double XS
        {
            get { return xs; }
            set { xs = Math.Max(0.00001, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private TrendChannel[] cacheTrendChannel = null;

        private static TrendChannel checkTrendChannel = new TrendChannel();

        /// <summary>
        /// Trend Channel of StandardDeviation of Movingaverage relation to RSI and ATR
        /// </summary>
        /// <returns></returns>
        public TrendChannel TrendChannel(double dev, int mALength, int rSILength, double xL, double xS)
        {
            return TrendChannel(Input, dev, mALength, rSILength, xL, xS);
        }

        /// <summary>
        /// Trend Channel of StandardDeviation of Movingaverage relation to RSI and ATR
        /// </summary>
        /// <returns></returns>
        public TrendChannel TrendChannel(Data.IDataSeries input, double dev, int mALength, int rSILength, double xL, double xS)
        {
            if (cacheTrendChannel != null)
                for (int idx = 0; idx < cacheTrendChannel.Length; idx++)
                    if (Math.Abs(cacheTrendChannel[idx].Dev - dev) <= double.Epsilon && cacheTrendChannel[idx].MALength == mALength && cacheTrendChannel[idx].RSILength == rSILength && Math.Abs(cacheTrendChannel[idx].XL - xL) <= double.Epsilon && Math.Abs(cacheTrendChannel[idx].XS - xS) <= double.Epsilon && cacheTrendChannel[idx].EqualsInput(input))
                        return cacheTrendChannel[idx];

            lock (checkTrendChannel)
            {
                checkTrendChannel.Dev = dev;
                dev = checkTrendChannel.Dev;
                checkTrendChannel.MALength = mALength;
                mALength = checkTrendChannel.MALength;
                checkTrendChannel.RSILength = rSILength;
                rSILength = checkTrendChannel.RSILength;
                checkTrendChannel.XL = xL;
                xL = checkTrendChannel.XL;
                checkTrendChannel.XS = xS;
                xS = checkTrendChannel.XS;

                if (cacheTrendChannel != null)
                    for (int idx = 0; idx < cacheTrendChannel.Length; idx++)
                        if (Math.Abs(cacheTrendChannel[idx].Dev - dev) <= double.Epsilon && cacheTrendChannel[idx].MALength == mALength && cacheTrendChannel[idx].RSILength == rSILength && Math.Abs(cacheTrendChannel[idx].XL - xL) <= double.Epsilon && Math.Abs(cacheTrendChannel[idx].XS - xS) <= double.Epsilon && cacheTrendChannel[idx].EqualsInput(input))
                            return cacheTrendChannel[idx];

                TrendChannel indicator = new TrendChannel();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.Dev = dev;
                indicator.MALength = mALength;
                indicator.RSILength = rSILength;
                indicator.XL = xL;
                indicator.XS = xS;
                Indicators.Add(indicator);
                indicator.SetUp();

                TrendChannel[] tmp = new TrendChannel[cacheTrendChannel == null ? 1 : cacheTrendChannel.Length + 1];
                if (cacheTrendChannel != null)
                    cacheTrendChannel.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheTrendChannel = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Trend Channel of StandardDeviation of Movingaverage relation to RSI and ATR
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TrendChannel TrendChannel(double dev, int mALength, int rSILength, double xL, double xS)
        {
            return _indicator.TrendChannel(Input, dev, mALength, rSILength, xL, xS);
        }

        /// <summary>
        /// Trend Channel of StandardDeviation of Movingaverage relation to RSI and ATR
        /// </summary>
        /// <returns></returns>
        public Indicator.TrendChannel TrendChannel(Data.IDataSeries input, double dev, int mALength, int rSILength, double xL, double xS)
        {
            return _indicator.TrendChannel(input, dev, mALength, rSILength, xL, xS);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Trend Channel of StandardDeviation of Movingaverage relation to RSI and ATR
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TrendChannel TrendChannel(double dev, int mALength, int rSILength, double xL, double xS)
        {
            return _indicator.TrendChannel(Input, dev, mALength, rSILength, xL, xS);
        }

        /// <summary>
        /// Trend Channel of StandardDeviation of Movingaverage relation to RSI and ATR
        /// </summary>
        /// <returns></returns>
        public Indicator.TrendChannel TrendChannel(Data.IDataSeries input, double dev, int mALength, int rSILength, double xL, double xS)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.TrendChannel(input, dev, mALength, rSILength, xL, xS);
        }
    }
}
#endregion
