#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Indicator;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Strategy;
#endregion

namespace NinjaTrader.Strategy
{ 
	#region Discription
	/* NinjaWorkShop1.0 : 01/01/2014		Coded by DerekPhelps22@gmail.com 
	   NinjaWorkshop1.1 : 01/30/2014		Corrected stop errors and made allowed strategy controls to exit trades when ATM active
		NOTE: This install came with a QuickStartGuide in the .Zip.  READ IT! to get the most out of this strategy
		NinjaWorkShop Strategy ::Demo version of production strategy I activly trade..(ie: vanilla indicators)
		This is a fully functional multi-control, multi-entry strategy.  Each entry is treated as a seperate trade position with its own ProfitStop and StopLoss setting. 
		It was written in a semi modular construct as a tool that I could quickly and easily plug in any indicator control for testing in a way that could compliment trigger
		signals of other indicators in a seamless manner and expose all parameters of indicators for tuning to specific instruments and time periods.  In this public version 
		I have included some industry standard indicators as well as a couple custom ones (The 'Pro' version has better indicators :-).  If you are a programmer you should be able to add additional indicators/controls is
		the manner that I have and not have to worry about breaking the trade management (stops/targets/multientry/etc) system already in place.
	
		If you are not a programmer and have a favorite indicator you trade with and wish to add to this workshop you may contact me with your requested change/trade-idea/upgrade
		to enhance NinjaWorkShop (or the upgraded NinjaWorkshopPro) to your specification.
		Some of its features as is are...
		* Entry conditions are signaled only if ALL active indicator entry conditions are met (AND function).  
		* Exit signals are generated if ANY active indicator condition is met (OR function) (Difficult to get in, quick to get out).
		* Optional handover to ATM strategy of choice. Upon triggered Entry ATM assumes full control until ATM exit returns control back to strategy
		The parameter operations are as follows...
	ENTRIES/EXITS
		In this demo iteration there are 7 main indicator controls and 5 minor entry limit methods of controlling entrys and exits.
		Main
		1) AutoTrendH 	:Enter/Exit on rising/falling trendlines and optionally trend line breaks
		2) MACD			:Enter/Exit on fast/slow MACD crossings or optionaly rising/falling Diffs
		3) Slopes	 	:Enter/Exit on user selected rising/falling indicator (SMA, EMA, LinReg, RSI) slopes with adjustable slope angle to reduce entry's during congestion.
		4) TrendChannel	:Enter/Exit on rising/falling TrendChannel lines
		5) Scalper		:Enter/Exit on Buy/Sell Scalper trigger
		6) EMA			:Enter/Exit on fast/slow EMA crossings
		7) RSI			:Enter/Exit on user selected RSI levels
		Minor
		1)	LongTrendLimit		: Limit entries only in direction of user selectable long term trend direction (with adjustable slope trade angle limit to reduce congestion entries)
		2)	PreviousBarLimit	: Only allow triggered entry is the previous bar closed in the direction of trigger.
		3)	ReEntryBars Delay 	: Prevent re-entrys for X number of bars after an exit. (prevents immediate re-entry if stopped out and indicators are giving an entry signal)
		4)	Re-entry prevention 	: Refuse re-entry in same direction until neutral or reverse trigger signal encountered.
		5)	Reverse Triggers 	: Reverse Entry/Exit signals (use if your settings come up with truly horrible results )
	MANAGE
		ATM Handover: If ATM strategy selected from list, upon strategy entry ATM assumes full control until exit (Only operative on Live or MarketReplay data)
			NOTE	: The following stop values are ignored if ATM management activated
		StopLoss	: Exits a position if price moves StopLoss pips/ticks in wrong direction.
		StopProfit	: Exits a position if price moves in desired direction StopProfit ticks.
		StopTrail	: Traiiling stop level set 'StopTrail' pips/ticks behind StopProfit level if reached.
		StopEven	: Creates StopLoss setting at breakeven if price moves StopEven ticks in desired direction (a Red or Green dot will appear on entry bar when stoploss has been set at BreakEven level.
		MaxEntries	: Open an independent trade up to MaxEntries for every new bar close more profitable then the previous entry.
	TRADING TIMES/TARGET
		You may optionaly enter a Start/Stop time to allow trading along with a price target to end trading for the session early.(Ninjas Session must be set to Daily sessions to work right)
		In the Trading Times parameters a StartTime of 0 disables this section.
		If you enter a StartTime, you must enter a StopTime and ProfitTarget value along with it.
	*/
	[Description("Multi-control/Multi entry strategy. TradeIdea: That some indicators are better at entries and some at exits. Create a strategy that allows easy testing of mix and match indicators for their purpose.  Design goal was to write in a modular way to allow flexability and relitive ease for future additions of other controls/indicators")]
	#endregion
		//
	public class NinjaWorkShop : Strategy, ICustomTypeDescriptor
    {
	#region Variables
		#region AtmHandover Variables
		//Allows for handover of trade management to your selected ATM strategy upon entry
			private string	atmStrategyId		= string.Empty;	// Variable to hold the atmStrategyId
			private string	orderId				= string.Empty; // Variable to hold the orderId
			private string	_ATMTemplate 		= "";
//				private bool	atmFullControl	= false; //Hand over full control to ATM strategy upon entry (strategy will not enter opposing trade until flat again).
			public class AtmConverter : StringConverter 
			{	public override bool GetStandardValuesSupported(ITypeDescriptorContext context) {	return true;	}
				public override bool GetStandardValuesExclusive(ITypeDescriptorContext context)	{	return false;	}
				public override StandardValuesCollection GetStandardValues(ITypeDescriptorContext context)
				{	string[] sarr = new string[0];
					foreach (StrategyData data in NinjaTrader.Db.Globals.Engine.GetObjectSet(typeof(StrategyData), "category = " + 2))
					{	if (data.Strategy is AtmStrategy)
						{	Array.Resize<string>(ref sarr, sarr.Length+1);	sarr[sarr.Length-1] = data.Strategy.Name;
					}	}
					foreach (StrategyData data in NinjaTrader.Db.Globals.Engine.GetObjectSet(typeof(StrategyData), "category = " + 1))
					{	if (data.Strategy is AtmStrategy)
						{	Array.Resize<string>(ref sarr, sarr.Length+1);	sarr[sarr.Length-1] = data.Strategy.Name;
					}	}
					return new StandardValuesCollection( sarr );
			}	};
		#endregion
		#region AutoTrendH Variables
			private AutoTrends		autoTrend	= AutoTrends.Disabled;	// Dropdown Control for enabling AutoTrendH indicator signals
				public enum AutoTrends		{	Disabled = 1,	Entrys	= 2,	Exits	= 3,	Both	= 4,	DisplayOnly	=	5,	}
			private bool 	alert			= true;		// Enables an audio and Alerts Log alert on trendline violation
			private bool 	enterBreaks		= true;		// Allow Long entrys if price breaks above downtrend resistance line and Shorts on price break below support trend line
			private bool 	showHistory		= true;		// Keeps historical trendline markers on the chart
			private int  	strength		= 7;		// Strength of trend line (smaller numbers draws shorter term trend lines (noiser)
			private int 	AT_Entry		= cDisabled;	private int AT_Exit	=cDisabled;		// Internal variables used in 'Triggers' region
		#endregion		
		#region EMA Xing Variables
			private EmaCrosses		emaCross	= EmaCrosses.Disabled;	// Dropdown Control for enabling EMA Crossing indicator signals
				public enum EmaCrosses		{	Disabled = 1,	Entrys	= 2,	Exits	= 3,	Both	= 4,	DisplayOnly	=	5,	}
			private EmaLimits		emaLimit	= EmaLimits.Exits;		// Dropdown Control for Only allow trades when price is above/below slow EMA plot
				public enum EmaLimits		{	Disabled = 1,	Entrys	= 2,	Exits	= 3,	Both	= 4,	}
			private	int		emaFast			= 8;
			private	int		emaSlow			= 32;
			private int		emaTrigger		= cFlat;	private int	EMA_Entry	= cDisabled;	private int	EMA_Exit=cDisabled;		// Internal variables used in 'Triggers' region
		#endregion
		#region MACD Variables
			private MacdCrosses		macdCross	= MacdCrosses.Disabled;	// Dropdown Control for enabling MACD indicator signals
				public enum MacdCrosses		{	Disabled = 1,	Entrys	= 2,	Exits	= 3,	Both	= 4,	DisplayOnly	=	5,	}
			private MacdSignals		macdSignal	= MacdSignals.Crosses;	// Dropdown Control for choosing type of MACD signals
				public enum MacdSignals		{	Crosses = 1,									Diffs	= 2		}	
			private bool	macdLimit		= false;		// 
			private int		macdFast		= 18;		// 
			private int 	macdSlow 		= 21;		// 
			private int 	macdSmooth		= 24;		// 
			private int  	macdMinDiff		= 0;		// Minimum difference (pips/ticks) in slow/fast MA's to allow entrys (reduce congestion trading)
			private int 	macdTrigger		= cFlat;	private int	MACD_Entry	= cDisabled;	private int	MACD_Exit	=cDisabled;	// Internal variables used in 'Triggers' region
		#endregion		
		#region Slopes Variables
			private SMASlopes		smaSlope	= SMASlopes.Disabled;		// Dropdown Control for enabling Slopes indicator signals
				public enum SMASlopes		{	Disabled = 1,	Entrys	= 2,	Exits	= 3,	Both	= 4,	DisplayOnly	=	5,			}
			private Slopes.Indicates	indicate	= Indicator.Slopes.Indicates.SMA;  
			private bool 	smaLimits		= false;
			private int		smaLookback		= 1;		// 
			private int 	smaPeriod 		= 40;		// 
			private int 	smaSmooth		= 30;		// 
			private int		tradeAngle		= 0;		// Minimum +/- SMA angle to allow trading
			private int		smaTrigger		= cFlat;	private int	SMA_Entry	= cDisabled;	private int	SMA_Exit=cDisabled;		// Internal variables used in 'Triggers' region
		#endregion		
		#region RSI Variables
			private RSI2s			rsi2		= RSI2s.Disabled;		// Dropdown Control for enabling RSI %b indicator signals
				public enum RSI2s			{	Disabled = 1,	Entrys	= 2,	Exits	= 3,	Both	= 4,	DisplayOnly	=	5,	}
			private bool	rsi2Limit		= false;		
			private int		rsi2Period		= 10;		
			private double 	rsi2Upper		= 55;		private double	rsi2Lower	= 45;		// Upper entry/exit threshold for Boll%b control for both triggers 
			private int 	rsi2Smooth 		= 2;		// 
			private int		RSI2_Entry	= cDisabled;	private int		RSI2_Exit	=cDisabled;	// Internal variables used in 'Triggers' region
			private int 	rsi2Trigger		= cFlat;
		#endregion		
		#region Scalper Variables
			private Scalpers		scalper		= Scalpers.Disabled;		// Dropdown Control for enabling Scalper indicator signals
				public enum Scalpers		{	Disabled = 1,	Entrys	= 2,	Exits	= 3,	Both	= 4,	DisplayOnly	=	5,	}
			private	int		sTrigger		= 0;
			private int		ofTrigger		= cFlat;	private int	S_Entry	= cDisabled;	private int	S_Exit=cDisabled;		// Internal variables used in 'Triggers' region
		#endregion		
		#region TrendChannel Variables
			private	TrendChannels	trendChannel= TrendChannels.Disabled;	// Dropdown Control for enabling TrendChannel indicator signals
				public enum TrendChannels	{	Disabled = 1,	Entrys	= 2,	Exits	= 3,	Both	= 4,	DisplayOnly	=	5,	}
			private ChannelLimits	channelLimit= ChannelLimits.Disabled;	// Dropdown Control for enabling ChannelLimit signals
				public enum ChannelLimits	{	Disabled = 1,	Entrys	= 2,	Exits	= 3,	Both	= 4,	DisplayOnly	=	5,	}
			private double 	dev 			= 1.5;		// Default setting for TrendChannel deviation (TrendCHannal Width used with ChannalLimit)
			private double 	xl 				= 51;		// MA angle (higher=harder to enter Long trade
			private double 	xs 				= 49;		// MA angle (lower = harder to enter Short trade
			private int 	maLength 		= 50; 		// Default setting for MALength
			private int 	rsiLength		= 30;
			private int		tcTrigger		= 0;		//private int  tcTmpTrigger = 0;
			private int		TC_Entry		= cDisabled; 	private int		TC_Exit  = cDisabled;	//Optional TrendChannel Trigger for validation price trending in TradeChannel
			private int		CL_Entry		= cDisabled;	private int		CL_Exit  = cDisabled;	//Optional ChannelLimit Trigger for validation of price outside of TrendChannel (breakout)
		#endregion		
		#region EntryLimits Variables
			private LongTrends		longTrend	= LongTrends.Disabled;	// Dropdown Control for enabling AutoTrendH indicator signals
				public enum LongTrends		{	Disabled = 1,	AboveBelow	= 2,	RiseFall	= 3,	Both	= 4,	DisplayOnly	=	5,	}
			private EntrysLimits	entrysLimit	= EntrysLimits.Both;	// Dropdown Control for enabling AutoTrendH indicator signals
				public enum EntrysLimits	{	Both = 1,		LongOnly	= 2,	ShortOnly	= 3,		}
			private int		longTrendPeriod = 0;		//Limits entrys in direction of SMA long trend only
			private int		longTrendAngle	= 0;		//Minimum steepness of long trend line to allow entrys (yellow portion of line)
			private int		longTrendDirection= 0;
			private int		entryDelay		= 0;			private int		delayCounter	= 0;		// Bars delay before entering a new trade after exit
			private int		LT_Entry		= cDisabled;	private int		LT_Exit	= cDisabled;	//Optional LongTermTrend trigger to hold direction of long term trend	
		#endregion		
		#region TradeTime Variables
			private int     startTime1		= 0;		//0930; // Time to start trading
			private int     stopTime1		= 0;		//1600; // Time to stop trading if target not reached (1600 = 04:00PM)
			private int     startTime2		= 0;		//0930; // Time to start trading
			private int     stopTime2		= 0;		//1600; // Time to stop trading if target not reached (1600 = 04:00PM)
			private double	profitTarget	= 0;		// Stop trading for the day if profitTarget reached
			private double  priorTradesCumProfit=0;	private double  profitThisSession=0;
		#endregion		
		#region Management/Stop Variables
			private TrailStops		trailStop	= TrailStops.Disabled;	// Dropdown Control for enabling MACD indicator signals
				public enum TrailStops		{	Disabled = 1,	Fixed	= 2,	ATR	= 3,	SMA	= 4,	}
			private bool 	atOppisite		= false;	// Reverse Entry/Exit trigger signals
			private bool	prevBarLimit	= false;	//Only allow entry if previous bar is in direction of entry (Good for preventing immidiate re-entry in wrong direction if position exited on a Stop)
			private int		PB_Limit		= cDisabled;//Variable to hold direction of last bar
			private bool 	prevent			= false;	// Dont allow re-entry until either neutral or oppisite trigger condition	
//			private bool 	predict			= false;	// Attempt to get 1 bar earlier entry by predicting where currency value will be on next bar
			private int 	quantity		= 0;
			private bool	trace			= false;	// Turn on tracking for trade progress in output window 
			private bool	stopMarkers	= true; 		// Turn on chart markers for stops display
			private int		stopEven		= 0;		// Move exit to breakeven after profitable stopEven pips/ticks 
			private int		stopProfit		= 0;		// Move stop up to entryprice +- StopProfit after profitable move of stopProfit pips/ticks 
			private int		stopTrailValue	= 0;		// TrailingStop behind StopProfit
			private int		stopTrailPeriod	= 0;		// Lookback period for ATR or SMA trailinig stop
			private double	stopTrailMult	= 0.0;		// ATR multiplier for trailinig stop
			private int		stopShock		= 0;		// Exits position if price moves stopShock pips/ticks against position in 1 minute.
			private double	previousMaxMove	= 0;		// tracking variable to move up trailing stop as position advances.
			private int		stopLoss		= 0;		// Initial stopLoss setting upon trade entry.
			//	private int		stopReverse		= 0;		// Reverse trade after stopReverse pips/ticks
			private int 	orderSize 		= 1;		// DefaultQuantity to trade
			private int		maxEntries		= 1;
			private int		posCtr			= 0;		// Counter to keep track of multiple entries
			private	int 	lastDir			= 0;
			private double	lastEntry 		= 0;		//tracking variable to only allow multi entrys if current bar is more profitable then last entry
		// Constants (makes coding more readable)
			const int 		cLong	= 1;  	const int cShort	=-1;	
			const int 		cRising	= 1;	const int cFalling	=-1;	const int cFlat	=0;	const int cDisabled	=3;	const int cWorking=4;
			const int 		cEntry	= 1;	const int cExit		=-1;
			const int 		stopped	= 0;	
				  bool 		debug	= false;
		#endregion		
	#endregion 
		//	
	#region Initialize
		protected override void Initialize()
		{ 	ExitOnClose					= false;
			CalculateOnBarClose 		= true;
			QuantityType 				= QuantityType.DefaultQuantity;
			quantity					= DefaultQuantity;
			orderSize					= DefaultQuantity;
			EntriesPerDirection			= 1;
			EntryHandling				= EntryHandling.UniqueEntries; 
			CalculateOnBarClose			= true;			
			if (trace)		TraceOrders	= true;
			// Activate indicators of active controls
			if (stopShock		> 0					)	Add(PeriodType.Minute, 1);
			if (autoTrend	!=AutoTrends.Disabled	)	Add(AutoTrendH(alert,showHistory,strength));
			if (scalper		!=Scalpers.Disabled		)	Add(ScalperH());
			if (smaSlope	!=SMASlopes.Disabled	)	Add(Slopes(indicate,smaLookback,smaPeriod,smaSmooth,tradeAngle,Indicator.Slopes.Views.Indicator));
			if (macdCross	!=MacdCrosses.Disabled	)	Add(MACD(macdFast,macdSlow,macdSmooth));
			if (LongTrend!=LongTrends.Disabled		)	Add(Slopes(Indicator.Slopes.Indicates.SMA,1,longTrendPeriod,1,longTrendAngle,Indicator.Slopes.Views.Indicator));
			if (emaCross	!=EmaCrosses.Disabled	) {	Add(EMA(emaSlow));	EMA(emaSlow).Plots[0].Pen.Color=Color.Red;Add(EMA(emaFast)); EMA(emaFast).Plots[0].Pen.Color=Color.Green;	}
			if (rsi2!=RSI2s.Disabled&&rsi2Period >0 ) {	Add(RSI(rsi2Period ,rsi2Smooth));	RSI(rsi2Period ,rsi2Smooth).Lines[0].Value=rsi2Lower;	RSI(rsi2Period ,rsi2Smooth).Lines[1].Value=rsi2Upper; 	}
			if ((trendChannel!=TrendChannels.Disabled)||(ChannelLimit!=ChannelLimits.Disabled))	Add(TrendChannel(dev,maLength,rsiLength,xl,xs));
			//Create StopLoss and StopProfits for each position (user selected in MaxPositions)
			for (int StopCnt = 1; StopCnt <= MaxEntries; StopCnt++) //Will loop through and create a SetStopLoss and SetStopProfit for each potential trade entry
			{	if ((stopEven>0)||(TrailStop!=TrailStops.Disabled))		{ SetStopLoss("L"+StopCnt,CalculationMode.Ticks,1000,true);	SetStopLoss("S"+StopCnt,CalculationMode.Ticks,1000,true);		}
				if  (stopLoss>0)										{ SetStopLoss("L"+StopCnt,CalculationMode.Ticks,stopLoss,true);		SetStopLoss("S"+StopCnt,CalculationMode.Ticks,stopLoss,true);	}	
				if ((stopProfit>0)&&(TrailStop==TrailStops.Disabled))	{ SetProfitTarget("L"+StopCnt,CalculationMode.Ticks,stopProfit);	SetProfitTarget("S"+StopCnt,CalculationMode.Ticks,stopProfit);	}
		}	}
	#endregion
		//
	protected override void OnBarUpdate()
	{
	#region PreProcess  //Only allow Trading if TradeTime/ProfitTarget/No PriceShock checks are passed
	//REMOVE THIS SECTION	//Used when backtesting my Forex pairs. My Jpy's dont adjust for pip difference
			if ( (Historical)&&((Instrument.FullName == "$USDJPY") || (Instrument.FullName=="$GBPJPY") || (Instrument.FullName=="$EURJPY") || (Instrument.FullName=="$AUDJPY") || (Instrument.FullName=="$NZDJPY")) )  
				{	DefaultQuantity=(int)(quantity*.01); 		}
		//STOP Price Shock	
		if (stopShock>0) 
		{// Prevent major moves against position using 1 minute intrabar granularity, Exits position if adverse move of StopShock pips/ticks since last main bar close
			if ( (stopShock>0) && (BarsInProgress == 1)	) //&& (myPosition(0)!= cFlat) )
			{	if ( (myPosition(0) == cLong)  && (Closes[0][0]-Closes[1][0]  > stopShock*TickSize)  )	
				{	if (trace)	Print("StopShock Exit@"+Time[0].ToString("G")+" BSE="+BarsSinceEntry(0,"",0)+" LEntryClose="+Closes[0][BarsSinceEntry(0,"",0)]+" Close[0]="+Closes[0][0]+" Stop="+(stopShock*TickSize));
					ExitLong("Shock","");	posCtr=0;
				}
				if ( (myPosition(0) == cShort) && (Closes[1][0]-Closes[0][0]  > stopShock*TickSize)  )	
				{	if (trace)	Print("StopShock Exit@"+Time[0].ToString("G")+" BSE="+BarsSinceEntry(0,"",0)+" SEntryClose="+Closes[0][BarsSinceEntry(0,"",0)]+" Close[0]="+Closes[0][0]+" Stop="+(stopShock*TickSize));	
					ExitShort("Shock","");	posCtr=0;
		}	}	}	
		if (BarsInProgress != 0)	return;		//We only want the primary bars processing from this point on.
	// TRADING TIME/TARGET CHECK	
			//If user has selected a startTime only allow trading if Time >StartTime && Time < stopTime && Profit<profitTarget
			if((!tradingAllowed())&&((startTime1>0)||(startTime2>0)) )
			{	if (myPosition(0)==cLong) 	ExitLong();	if (myPosition(0)==cShort) 	ExitShort();
				if ((trace)&&((startTime1>0)||(startTime1>0)))	Print("Trade Time/Target Violation: "+Time[0].ToString("G")+" must be >"+(startTime1).ToString("##:##")+"and <"+(stopTime1).ToString("##:##")+" Profit is "+profitThisSession.ToString("C")+" Target is $"+profitTarget);
				DrawDiamond(CurrentBar.ToString(), true,0,Low[0],Color.Yellow);
				return;	
			}	
			if ((trace)&&((startTime1>0)||(startTime1>0)) )	Print("Trade Time/Target OK:"+Time[0].ToString("G")+" Session Profit:"+profitThisSession.ToString("C"));
	// ENTRY DELAY  						
			delayCounter=delayCounter+1;
			if (myPosition(0)!=cFlat) delayCounter=0;	//reset re-entry delay any time we have a position open
	#endregion
		//
	#region LoadTriggers
		//LOAD TRIGGERS (Load entry/exit triggers used later in the Entrys/Exits section
		#region AutoTrendH Triggers
			if (autoTrend!=AutoTrends.Disabled&&autoTrend!=AutoTrends.DisplayOnly) 
			{	int trendDirection=AutoTrendH(alert,showHistory,strength).Direction; double trendPrice=AutoTrendH(alert,showHistory,strength).TrendPrice;	
				AT_Entry=cFlat;  AT_Exit=cFlat;		//Initialize triggers Flat and let next 2 lines set them correctly as condition warrents
				if ((!enterBreaks)&&(trendDirection==cRising  ))	{ 	if(Close[0]>trendPrice) 	{AT_Entry=cLong;  AT_Exit=cLong; }	else {AT_Entry=cFlat; AT_Exit=cShort;}	}
				if ((!enterBreaks)&&(trendDirection==cFalling ))	{ 	if(Close[0]<trendPrice) 	{AT_Entry=cShort; AT_Exit=cShort;}	else {AT_Entry=cFlat; AT_Exit=cLong; } 	}
				if (( enterBreaks)&&(Close[0]>trendPrice))			{AT_Entry=cLong;  AT_Exit=cLong;}		
				if (( enterBreaks)&&(Close[0]<trendPrice)) 			{AT_Entry=cShort; AT_Exit=cShort;} //preload triggers with current AutoTrend bias
				switch(AutoTrend)	//
				{	case AutoTrends.Disabled:		{AT_Entry=cDisabled; 	AT_Exit=cDisabled;		}	break;
					case AutoTrends.Entrys: 		{						AT_Exit=cDisabled;		}	break;
					case AutoTrends.Exits: 			{AT_Entry=cDisabled;							}	break;
					case AutoTrends.Both: 			{												}	break;
			}	}	
		#endregion
		#region EMACross Triggers
			if (emaCross!=EmaCrosses.Disabled&&emaCross!=EmaCrosses.DisplayOnly) 
			{	if (EMA(emaFast)[0]>EMA(emaSlow)[0]) emaTrigger=cLong; else emaTrigger=cShort;
				bool emaPriceAbove = (Close[0]>EMA(emaSlow)[0]);
				switch(EmaCross)	//
				{	case EmaCrosses.Disabled:		{EMA_Entry=cDisabled; 	EMA_Exit=cDisabled;		}	break;
					case EmaCrosses.Entrys:			{EMA_Entry=emaTrigger;	EMA_Exit=cDisabled;		}	break;
					case EmaCrosses.Exits: 			{EMA_Entry=cDisabled;	EMA_Exit=emaTrigger;	}	break;
					case EmaCrosses.Both: 			{EMA_Entry=emaTrigger;	EMA_Exit=emaTrigger;	}	break;
				}
				switch(EmaLimit)	//
				{	case EmaLimits.Disabled:		{																																}	break;
					case EmaLimits.Entrys:			{if((EMA_Entry==cLong && !emaPriceAbove)||(EMA_Entry==cShort && emaPriceAbove))	EMA_Entry=cFlat;								}	break;
					case EmaLimits.Exits: 			{if((myPosition(0)==cLong && !emaPriceAbove))	EMA_Exit=cShort; if((myPosition(0)==cShort && emaPriceAbove))	EMA_Exit=cLong;	}	break;
					case EmaLimits.Both: 			{if((EMA_Entry==cLong && !emaPriceAbove)||(EMA_Entry==cShort && emaPriceAbove))	EMA_Entry=cFlat; 
													 if((myPosition(0)==cLong && !emaPriceAbove))	EMA_Exit=cShort; if((myPosition(0)==cShort && emaPriceAbove))	EMA_Exit=cLong;	}	break;
			}	}
		#endregion
		#region MacdCross Triggers
			if (macdCross!=MacdCrosses.Disabled&&macdCross!=MacdCrosses.DisplayOnly) 
			{	if (macdSignal==MacdSignals.Crosses)	{	if (MACD(macdFast,macdSlow,macdSmooth)[0]>MACD(macdFast,macdSlow,macdSmooth).Avg[0])		{	macdTrigger=cLong;} else {macdTrigger=cShort;}};
				if (macdSignal==MacdSignals.Diffs)		{	if (MACD(macdFast,macdSlow,macdSmooth).Diff[0]>MACD(macdFast,macdSlow,macdSmooth).Diff[1])	{	macdTrigger=cLong;} else {macdTrigger=cShort;}};
				bool avgRising=((MACD(macdFast,macdSlow,macdSmooth).Avg[0]>MACD(macdFast,macdSlow,macdSmooth).Avg[1]));
				bool diffRising=((MACD(macdFast,macdSlow,macdSmooth).Diff[0]>MACD(macdFast,macdSlow,macdSmooth).Diff[1]));
				switch(MacdCross)	//
				{	case MacdCrosses.Disabled:		{MACD_Entry=cDisabled; 	MACD_Exit=cDisabled;	}	break;
					case MacdCrosses.Entrys:		{MACD_Entry=macdTrigger;MACD_Exit=cDisabled;	}	break;
					case MacdCrosses.Exits: 		{MACD_Entry=cDisabled;	MACD_Exit=macdTrigger;	}	break;
					case MacdCrosses.Both: 			{MACD_Entry=macdTrigger;MACD_Exit=macdTrigger;	}	break;
				}	
				if (macdLimit&&macdSignal==MacdSignals.Diffs)	{if (MACD_Entry==cLong&&!avgRising)MACD_Entry=cFlat;if (MACD_Entry==cShort&&avgRising)MACD_Entry=cFlat;}
				if (macdLimit&&macdSignal==MacdSignals.Crosses)	{if (MACD_Entry==cLong&&!diffRising)MACD_Exit=cShort;if (MACD_Entry==cShort&&diffRising)MACD_Exit=cLong;}
				if ( (macdMinDiff>0)&&(macdMinDiff*TickSize>Math.Abs(MACD(macdFast,macdSlow,macdSmooth).Diff[0])) ) MACD_Entry=cFlat;	//If minimum Diff enabled (either Exits or Both)only allow entrys > macdMinDiff
			}
		#endregion
		#region RSI2 Triggers 
			if ((rsi2!=RSI2s.Disabled && rsi2Period>0 )&&(rsi2!=RSI2s.DisplayOnly) )
			{	RSI2_Entry=cFlat;		RSI2_Exit=cFlat;
				double RSI2Value=RSI(rsi2Period,rsi2Smooth).Avg[0];
				if( RSI2Value >=rsi2Upper )	{	RSI2_Entry=cLong;	RSI2_Exit=cLong;}	else	if( RSI2Value <=rsi2Lower ) 	{	RSI2_Entry=cShort;	RSI2_Exit=cShort; 	}	
				if (rsi2Limit) {if (myPosition(0)==cLong&&RSI2Value<rsi2Upper)RSI2_Exit=cShort; if (myPosition(0)==cShort&&RSI2Value>rsi2Lower)RSI2_Exit=cLong; 		}
			switch(Rsi2)	//
				{		case RSI2s.Disabled:	{	RSI2_Entry=cDisabled; 		RSI2_Exit=cDisabled;		}	break;
						case RSI2s.Entrys:		{								RSI2_Exit=cDisabled;		}	break;
						case RSI2s.Exits: 		{	RSI2_Entry=cDisabled;									}	break;
						case RSI2s.Both:	 	{															}	break;
			}	}
		#endregion	
		#region Scalper Triggers
			if (scalper!=Scalpers.Disabled&&scalper!=Scalpers.DisplayOnly) 
			{	int scalpDirection	= ScalperH().Direction;
//				sTrigger=cFlat;
				if (scalpDirection==cLong   && Close[0]>ScalperH().TriggerLow)	{ sTrigger=cLong; 	}
				if (scalpDirection==cShort	&& Close[0]<ScalperH().TriggerHigh) { sTrigger=cShort;	}
				switch(Scalp)	//
				{	case Scalpers.Disabled:			{S_Entry=cDisabled; 	S_Exit=cDisabled;		}	break;
					case Scalpers.Entrys: 			{S_Entry=sTrigger;		S_Exit=cDisabled;		}	break;
					case Scalpers.Exits: 			{S_Entry=cDisabled;		S_Exit=sTrigger;		}	break;
					case Scalpers.Both: 			{S_Entry=sTrigger;		S_Exit=sTrigger;		}	break;
			}	}
		#endregion
		#region Slopes Triggers
			if ((smaSlope!=SMASlopes.Disabled && smaPeriod>0)&&(smaSlope!=SMASlopes.DisplayOnly) )
			{	smaTrigger= (int)Slopes(indicate,smaLookback,smaPeriod,smaSmooth,tradeAngle,Indicator.Slopes.Views.Indicator).Trigger[0];		// Slopes.Trigger returns 1)cRising(green), 0)cFlat(yellow) or -1)cFalling(red)
				double	smaValue 	= Slopes(indicate,smaLookback,smaPeriod,smaSmooth,tradeAngle,Indicator.Slopes.Views.Indicator)[0];	
				switch(SMASlope)	//
				{	case SMASlopes.Disabled:			{SMA_Entry=cDisabled; 	SMA_Exit=cDisabled;		}	break;
					case SMASlopes.Entrys:				{SMA_Entry=smaTrigger; 	SMA_Exit=cDisabled;		}	break;
					case SMASlopes.Exits: 				{SMA_Entry=cDisabled;	SMA_Exit=smaTrigger;	}	break;
					case SMASlopes.Both: 				{SMA_Entry=smaTrigger;	SMA_Exit=smaTrigger;	}	break;
				}//If SMALimits enabled, dont allow short entrys above SMA or long entrys below SMA && Exit if price on wrong side of SMA
				if ( (smaLimits)&&((smaTrigger ==cRising && Close[0]<smaValue )||(smaTrigger ==cFalling && Close[0]>smaValue )) )	{ SMA_Entry =cFlat;		}	
				if ( (smaLimits)&&(myPosition(0)==cLong) &&(Close[0]<smaValue ) )	{ SMA_Exit =cShort;	}	if ( (smaLimits)&&(myPosition(0)==cShort)&&(Close[0]>smaValue ) )	{ SMA_Exit =cLong ;	}	
			}
		#endregion
		#region TrendChannel Triggers
			if (trendChannel!=TrendChannels.Disabled&&trendChannel!=TrendChannels.DisplayOnly)	
			{	tcTrigger = TrendChannel(dev,maLength,rsiLength,xl,xs).TradeTrigger;	// TrendChannel.TradeTriggers returns 1)cRising(green dot) 0)cFlat(no dot) or -1)cFalling(red dot)
				switch(Trend_Channel)	//
				{	case TrendChannels.Disabled:	{TC_Entry=cDisabled; 	TC_Exit=cDisabled;	}	break;
					case TrendChannels.Entrys:	 	{TC_Entry=tcTrigger; 	TC_Exit=cDisabled;	}	break;
					case TrendChannels.Exits: 		{TC_Entry=cDisabled;	TC_Exit=tcTrigger;	}	break;
					case TrendChannels.Both: 		{TC_Entry=tcTrigger;	TC_Exit=tcTrigger;	}	break;
			}	}
		#endregion
		#region	ChannelLimit Triggers	
			if (channelLimit!=ChannelLimits.Disabled&&channelLimit!=ChannelLimits.DisplayOnly)	
			{	double channelMedian=TrendChannel(dev,maLength,rsiLength,xl,xs).MidLine[0]; CL_Entry=cFlat;	CL_Exit=cFlat;
				if (Close[0]>=TrendChannel(dev,maLength,rsiLength,xl,xs).Upper[0])	CL_Entry=cLong; 	if (Close[0]<=TrendChannel(dev,maLength,rsiLength,xl,xs).Lower[0])	CL_Entry=cShort; 
				if ( (myPosition(0)==cLong&&Close[0]<=channelMedian) ) 				CL_Exit =cShort;	if(myPosition(0)==cShort&&Close[0]>channelMedian) 					CL_Exit =cLong;
				switch(ChannelLimit)	//This is an optional test to allow entry/exit signals if close breaks out of trend channel (open trades) or crosses median (exit trades)
				{		case ChannelLimits.Disabled:	{CL_Entry=cDisabled;	CL_Exit=cDisabled; 	}	break;	//Boolian triggers Entry:True=Allow trade entry, Exit:False=Do not trigger exit
						case ChannelLimits.Entrys:	 	{				 		CL_Exit=cDisabled;	}	break;
						case ChannelLimits.Exits: 		{CL_Entry=cDisabled;			 			}	break;
						case ChannelLimits.Both: 		{											}	break;
			}	}
		#endregion	
		#region LongTermTrend Limit //:Only allow entrys in direction of LongTerm trend
			if (LongTrend!=LongTrends.Disabled&&LongTrend!=LongTrends.DisplayOnly)
			{	LT_Entry=cFlat;	LT_Exit=cFlat;
				double  LTValue=	Slopes(Indicator.Slopes.Indicates.SMA,1,longTrendPeriod,3,LongTrendAngle,Indicator.Slopes.Views.Indicator)[0];
				int		LTTrigger=	(int)Slopes(Indicator.Slopes.Indicates.SMA,1,longTrendPeriod,3,LongTrendAngle,Indicator.Slopes.Views.Indicator).Trigger[0];
				if (LongTrend==LongTrends.RiseFall  )	{ 												LT_Entry=LTTrigger;	}	//Entry allowed in direction of slope (rising or falling)
				if (LongTrend==LongTrends.AboveBelow)	{ if(Close[0]>LTValue) 							LT_Entry=cLong;		}	//Long  Entry allowed if price above slope
				if (LongTrend==LongTrends.AboveBelow)	{ if(Close[0]<LTValue) 							LT_Entry=cShort; 	}	//Short Entry allowed if price below slope
				if (LongTrend==LongTrends.Both)			{ if((Close[0]>LTValue)&&(LTTrigger==cLong )) 	LT_Entry=cLong;		}	//Long  Entry allowed if slope rising AND price above slope
				if (LongTrend==LongTrends.Both)			{ if((Close[0]<LTValue)&&(LTTrigger==cShort)) 	LT_Entry=cShort; 	}	//Short Entry allowed if slope falling AND price below slope
			}//Previous Direction Limit : Only allow entrys if previous bar is in direction of entry
			if (PreviousBarLimit)  //If user has enabled Previous Bar Limit control
			{	PB_Limit=cFlat;
				if(Close[0]>Close[1]) PB_Limit=cLong;	if(Close[0]<Close[1]) PB_Limit=cShort;	
			}
		#endregion
		#region Trigger Validation
			//User selected indicator triggers must ALL test true (AND function) to trigger a long or short entry. ANY short trigger (OR function) will trip an exit trigger.
			if (trace) Print(	Time[0].ToString("G")+" Position="+myPosition(0)+" S_Entry="+S_Entry+" S_Exit="+S_Exit+" AT_Entry="+AT_Entry+" AT_Exit="+AT_Exit+" TC_Entry="+TC_Entry+" TC_Exit="+TC_Exit+" SMA_Entry="+SMA_Entry+" SMA_Exit="+SMA_Exit+" MACD_Entry="+MACD_Entry+" MACD_Exit="+MACD_Exit+" CLEntry="+CL_Entry+" CL_Exit="+CL_Exit+" S_Entry="+S_Entry+" S_Exit="+S_Exit+" LT_Entry="+LT_Entry+" PB_Limit="+PB_Limit);
			//Print(	Time[0].ToString("G")+" Position="+myPosition(0)+" SW_Entry="+SW_Entry+" SW_Exit="+S_Exit+" S_Entry="+S_Entry+" S_Exit="+S_Exit+" AT_Entry="+AT_Entry+" AT_Exit="+AT_Exit+" TC_Entry="+TC_Entry+" TC_Exit="+TC_Exit+" SMA_Entry="+SMA_Entry+" SMA_Exit="+SMA_Exit+" MACD_Entry="+MACD_Entry+" MACD_Exit="+MACD_Exit+" CLEntry="+CL_Entry+" CL_Exit="+CL_Exit+" LT_Entry="+LT_Entry+" PB_Limit="+PB_Limit);
			if ( (TC_Entry==cDisabled&&EMA_Entry==cDisabled&&SMA_Entry==cDisabled&&RSI2_Entry==cDisabled&&AT_Entry==cDisabled&&CL_Entry==cDisabled&&S_Entry==cDisabled&&MACD_Entry==cDisabled&&LT_Entry==cDisabled) )
			{	Print("You must have at least one valid(not disabled) entry control enabled to trade!");	return;	}
			//Set Entry and Exit triggers based upon active controls outputs.  ALL active entry triggers must agree to trigger entry/ ANY active exit trigger will exit trade
			bool longEntryTrigger  =((TC_Entry==cDisabled||TC_Entry==cLong )&&(EMA_Entry==cDisabled||EMA_Entry==cLong )&&(SMA_Entry==cDisabled||SMA_Entry==cLong )&&(RSI2_Entry==cDisabled||RSI2_Entry==cLong  )&&(AT_Entry==cDisabled||AT_Entry==cLong )&&(CL_Entry==cDisabled||CL_Entry==cLong )&&(S_Entry==cDisabled||S_Entry==cLong )&&(MACD_Entry==cDisabled||MACD_Entry==cLong )&&(LT_Entry==cDisabled||LT_Entry==cLong )&&(PB_Limit==cDisabled||PB_Limit==cLong ) );
			bool shortEntryTrigger =((TC_Entry==cDisabled||TC_Entry==cShort)&&(EMA_Entry==cDisabled||EMA_Entry==cShort)&&(SMA_Entry==cDisabled||SMA_Entry==cShort)&&(RSI2_Entry==cDisabled||RSI2_Entry==cShort )&&(AT_Entry==cDisabled||AT_Entry==cShort)&&(CL_Entry==cDisabled||CL_Entry==cShort)&&(S_Entry==cDisabled||S_Entry==cShort)&&(MACD_Entry==cDisabled||MACD_Entry==cShort)&&(LT_Entry==cDisabled||LT_Entry==cShort)&&(PB_Limit==cDisabled||PB_Limit==cShort) );
			bool longExitTrigger   =((TC_Exit==cShort||EMA_Exit==cShort||SMA_Exit==cShort|| RSI2_Exit==cShort||AT_Exit==cShort||CL_Exit==cShort||S_Exit==cShort||MACD_Exit==cShort||LT_Exit==cShort));
			bool shortExitTrigger  =((TC_Exit==cLong ||EMA_Exit==cLong ||SMA_Exit==cLong || RSI2_Exit==cLong ||AT_Exit==cLong ||CL_Exit==cLong ||S_Exit==cLong ||MACD_Exit==cLong ||LT_Exit==cLong ) );
			//Print(Time[0].ToString("G")+" Pos="+myPosition(0)+"LastDir:"+lastDir+" LongEntry="+longEntryTrigger+"  ShortEntry="+shortEntryTrigger+"  LongExit="+longExitTrigger+"  ShortExit="+shortExitTrigger); Print(" ");
			//Reverse Entry/Exit triggers on user select
			if(atOppisite)	
			{	if(longEntryTrigger){longEntryTrigger=false;shortEntryTrigger=true;} else if(shortEntryTrigger){shortEntryTrigger=false;longEntryTrigger=true;}
				if(longExitTrigger ){longExitTrigger =false;shortExitTrigger =true;} else if(shortExitTrigger ){shortExitTrigger =false;longExitTrigger =true;}
			}//Dont allow re-entrys in same direction until triggers flat or in oppisite direction of last entry	
			if((prevent)&&(myPosition(0)==cFlat)){ if(!longEntryTrigger&&!shortEntryTrigger)lastDir=cFlat;if(lastDir==cLong&&longEntryTrigger)longEntryTrigger=false;if(lastDir==cShort&&shortEntryTrigger)shortEntryTrigger=false;}
			//Only allow entrys in user selected directions
			if(entrysLimit==EntrysLimits.LongOnly)shortEntryTrigger=false;	if(entrysLimit==EntrysLimits.ShortOnly)longEntryTrigger=false;
		#endregion
	#endregion	
		//
	#region EntrysExits	
		// If user has selected an EntryDelay then Dont allow entry if within X bars of last exit(prevents immediate re-entry in same direction if stopped out)
		if ((entryDelay>0)&&((myPosition(0)==cFlat)||(BarsSinceEntry()<=1))&&(delayCounter < entryDelay)) return;	
		//	Print("MyPos:"+myPosition(0)+" LET:"+longEntryTrigger+" SET:"+shortEntryTrigger+" LXT:"+longExitTrigger+" SXT:"+shortExitTrigger+" PosCtr:"+posCtr+"MaxPos:"+maxEntries);
	//ENTRYS		
		//LongEntry
		if ((longEntryTrigger) && (!longExitTrigger) )  			// if all active indicators show long AND there are no active exit long indicators, Enter Long position.
		{	if (myPosition(0)!=cLong) { posCtr=0; lastEntry=0; } 	//If this is first long entry, reset counters 
			if ( (posCtr<maxEntries)&&(Close[0]>lastEntry))			//Open another Long position if positions less then max entries and current close is higher then last open 
			{	if ( (stopLoss>0)||(stopEven>0)||(TrailStop!=TrailStops.Disabled) )		{	setMultiStop("L","Ticks",maxEntries,0);	}
				lastEntry=Close[0];		posCtr=posCtr+1;	previousMaxMove=0;	delayCounter=0; lastDir= cLong;
				if(_ATMTemplate=="")	EnterLong("L"+posCtr);	else	if (myPosition(0)!=cLong){atmCommand("BuyLong");}
				return;	
		}	}	
		else
		//ShortEntry	
		if ((shortEntryTrigger)&&(!shortExitTrigger) ) 				// if all active indicators show Short AND there are no active exit short indicators, Enter Short position.
		{	if (myPosition(0)!=cShort) { posCtr=0;	lastEntry=0; }
			if ( (posCtr<maxEntries)&&((Close[0]<lastEntry)||(lastEntry==0)))	
			{	if ( (stopLoss>0)||(stopEven>0)||(TrailStop!=TrailStops.Disabled) ) 	{	setMultiStop("S","Ticks",maxEntries,0);	}
				lastEntry=Close[0];		posCtr=posCtr+1;	previousMaxMove=0;	delayCounter=0; lastDir=cShort;
				if(_ATMTemplate=="")	EnterShort("S"+posCtr);	else	if (myPosition(0)!=cShort){atmCommand("SellShort");	}
				return;
		}	}
	//EXITS
		if ( myPosition(0)==cLong  && longExitTrigger )  			//If Long and LongExit trigger, exit regular or ATM position
		{	if(_ATMTemplate=="")	{ExitLong(); posCtr=0; }		else	{atmCommand("Exit"); posCtr=0;	}	}
		else
		if ( myPosition(0)==cShort && shortExitTrigger)  			//If Short and ShortExit trigger, exit regular or ATM position
		{	if(_ATMTemplate=="")	{ExitShort();posCtr=0; }		else	{atmCommand("Exit"); posCtr=0;	}	}	
	#endregion
		//
	#region StopsManage
	//STOPS MANAGE
	//StopLoss : Exits postion on negitive excursion of StopLoss pips
		if (stopLoss >0 ) //Note: this function moved to Initialize section	
		{	}
	//StopProfit : Exits position when profitable StopProfit pips from entry (unless StopTrail has been set)
		if (stopProfit >0 ) //Note: this function moved to Initialize section	
		{	}
		//
	//StopTrail : If minimum StopProfit level reached, set and maintain a trailing stop StopTrail pips behind price (and draw dot on screen showing current StopTrail position) 
		if ((TrailStop!=TrailStops.Disabled))	//If user has set a TrailingStop value...
		{	double trailExitPrice=0;
			switch(TrailStop)	//This is an optional test to set entry/exit variables only allow entries or exits if price is trending outside of trend channel
			{	case TrailStops.Fixed:	{if(myPosition(0)==cLong) trailExitPrice=Close[0]-(StopTrailValue*TickSize);  if(myPosition(0)==cShort) trailExitPrice=Close[0]+(StopTrailValue*TickSize);		}	break;
				case TrailStops.ATR:	{if(myPosition(0)==cLong) trailExitPrice=Close[0]-(ATR(30)[0]*stopTrailMult); if(myPosition(0)==cShort) trailExitPrice=Close[0]+(ATR(30)[0]*stopTrailMult);		}	break;
				case TrailStops.SMA:	{trailExitPrice=SMA(stopTrailPeriod)[0];																														}	break;
			}		
			if ( (myPosition(0)==cLong)&&((Close[0]-Position.AvgPrice >= stopProfit*TickSize)||stopProfit==0) )		//...and price has reached minimum trailing distance from entry, then set trail stop
			{	if ( ((Close[0]-Position.AvgPrice)> previousMaxMove)||(stopProfit==0&&previousMaxMove==0)) 			//if Fixed or ATR method, keep moving up stop as price increases
				{	previousMaxMove=Close[0]-Position.AvgPrice;
					if (trace)	Print("Set TrailingStop:Long#"+posCtr+" @"+Time[0].ToString("G")+" to "+trailExitPrice );
					setMultiStop("L","Price",posCtr,trailExitPrice);
					if (stopMarkers) DrawDot("Dot"+(CurrentBar), true, 0,trailExitPrice,Color.Blue); //Draw a Dot on entry bar where Stop has been moved to TrailingStop
			}	}	
			if ( (myPosition(0)==cShort)&&((Position.AvgPrice-Close[0] >= stopProfit*TickSize)||stopProfit==0) )
			{	if ( ((Position.AvgPrice-Close[0])> previousMaxMove)||(stopProfit==0&&previousMaxMove==0)) 
				{	previousMaxMove=Position.AvgPrice-Close[0];
					if (trace)	Print("Set TrailingStop:Short#"+posCtr+" @"+Time[0].ToString("G")+" to "+trailExitPrice  );
					setMultiStop("S","Price",posCtr,trailExitPrice );
					if (stopMarkers) DrawDot("Dot"+(CurrentBar), true, 0,trailExitPrice ,Color.Blue);
		}	}	}
		//			
	//StopEven : If profitable by StopEven pips, set exit stop at breakeven pips.
		if ( (stopEven>0) && (previousMaxMove==0) && (posCtr!=0)	)	//If price closes more then stopEven ticks, move stopLoss(s) to breakeven unless TrailingStop activated
		{	if ( (myPosition(0)==cLong)&&((Close[0]-Position.AvgPrice) >= stopEven*TickSize) )
			{	if (trace)	Print("Set BreakEvenStop:Long#"+posCtr+" @"+Time[0].ToString("G")+" to "+Position.AvgPrice );
				setMultiStop("L","Price",posCtr,Position.AvgPrice);		
				if (stopMarkers) DrawDot(CurrentBar.ToString(), true, BarsSinceEntry(0,"",0),Position.AvgPrice,Color.Yellow); //Draw a Dot on entry bar when Stop has been moved to breakeven
			}
			if ( (myPosition(0)==cShort &&( (Position.AvgPrice)-Close[0]) >= stopEven*TickSize) )
			{	if (trace)	Print("Set BreakEvenStop:Short#"+posCtr+" @"+Time[0].ToString("G")+" to "+Position.AvgPrice );
				setMultiStop("S","Price",posCtr,Position.AvgPrice);
				if (stopMarkers) DrawDot(CurrentBar.ToString(), true, BarsSinceEntry(0,"",0),Position.AvgPrice,Color.Yellow);
		}	}
		//
		#endregion
		//
	}//ENDOnBarUpdate()
		//
	#region UserDefinedMethods
	//	
		protected override void OnTermination() //Close any open ATM orders is strategy is disabled
		{	if(atmStrategyId != string.Empty)	AtmStrategyClose(atmStrategyId);	}
		//
		private int atmCommand(string command) // Process ATM Buy/Sell/Position commands
		{	if (Historical||_ATMTemplate=="") {return cFlat;}
			string[] orderStatus=new string[3];	string[] strategyStatus=new string[3];
			if (orderId.Length>0)	orderStatus=GetAtmStrategyEntryOrderStatus(orderId);
			if (orderStatus.GetLength(0)>0&&(orderStatus[2]=="Filled"||orderStatus[2]=="Cancelled"||orderStatus[2]=="Rejected"))	{orderId 		= string.Empty;}// If the order state is terminal, clear the order id value
			if (atmStrategyId.Length>0&&orderId==string.Empty&&GetAtmStrategyMarketPosition(atmStrategyId)==Cbi.MarketPosition.Flat){atmStrategyId	= string.Empty;}// If no pending ATM orders and atm market flat, clear StrategyID
			if (trace)	Print("ATMCommand="+command+" OrderId="+orderId+" StrategyId="+atmStrategyId);
			//Print("ATMCommand="+command+" OrderId="+orderId+" StrategyId="+atmStrategyId);
			if (command=="Position")	//then return the position of current ATM strategy
			{	if(atmStrategyId==string.Empty) return cFlat; 	
				else if	(GetAtmStrategyMarketPosition(atmStrategyId)==Cbi.MarketPosition.Flat)  return cFlat;
				else if	(GetAtmStrategyMarketPosition(atmStrategyId)==Cbi.MarketPosition.Long)  return cLong;
				else if	(GetAtmStrategyMarketPosition(atmStrategyId)==Cbi.MarketPosition.Short) return cShort;
				else return cFlat;
			}
			//Enter ATM Position
			if ((command=="BuyLong")&&(orderId==string.Empty))		//if command is 'Buy' and no active ATM order, process ATM long entry
			{	if (atmStrategyId!=string.Empty) {AtmStrategyClose(atmStrategyId); atmStrategyId="";} //If buy order while in short positon, close short first
				atmStrategyId = GetAtmStrategyUniqueId();	orderId = GetAtmStrategyUniqueId(); 
				AtmStrategyCreate(OrderAction.Buy,		OrderType.Market,0,0,TimeInForce.Gtc,orderId,_ATMTemplate,atmStrategyId);	}
			if ((command=="SellShort")&&(orderId==string.Empty))	//if command is 'Sell' and no active ATM order, process ATM short entry
			{	if (atmStrategyId!=string.Empty) {AtmStrategyClose(atmStrategyId); atmStrategyId="";}//If sell order while in long positon, close long first
				atmStrategyId = GetAtmStrategyUniqueId();	orderId = GetAtmStrategyUniqueId(); 
				AtmStrategyCreate(OrderAction.SellShort,OrderType.Market,0,0,TimeInForce.Gtc,orderId,_ATMTemplate,atmStrategyId);	}	
			//Exit ATM Position
			if ((command=="Exit")&&(orderId==string.Empty)&&(atmStrategyId!=string.Empty))	//if command is 'Sell' and no active ATM order, process ATM strategy short entry
			{	AtmStrategyClose(atmStrategyId); atmStrategyId="";		}
			return cWorking;  //If everything falls through
		}//ENDatmCommand()
		//
		private int myPosition(int posNbr)
		{// A cleaner way of coding position direction. Use posNbr if writing strategy with multiple position entries
			if ((Positions[posNbr].MarketPosition==MarketPosition.Long)			||(atmCommand("Position")==cLong ))	return cLong;
			else if ((Positions[posNbr].MarketPosition==MarketPosition.Short)	||(atmCommand("Position")==cShort))	return cShort;
			else if ((Positions[posNbr].MarketPosition==MarketPosition.Flat)	||(atmCommand("Position")==cFlat ))	return cFlat;  
			return cFlat;
		}//ENDmyPosition()
		//	
		private void setMultiStop(string Name,string Mode,int Cnt,double Value) //Loop to cycle through and set all StopLoss and StopEven positions
		{	if (myPosition(0)==cFlat)		posCtr=0;
			if ((Value==0)&&(stopLoss>0))	Value=stopLoss; 
			if ((Value==0)&&(stopLoss==0))	Value=1000;	//If not auto settng for StopLoss then reset SetStopLoss to a default state for stopeven and stoptrail
			if (trace)	Print("MultiStopCalled@ "+Time[0].ToString("yyyy-MM-dd")+" @ "+Time[0].ToString("hh:mm:ss")+"Name:"+Name+" Mode:"+ Mode+" "+" Cnt:"+Cnt+" "+" Setting:"+Value);
			for (int I = 1; I <= Cnt; I++) //Will loop through and create a SetStopLoss and SetStopProfit for each trade entry
			{	if (Mode=="Ticks")	SetStopLoss(Name+I,CalculationMode.Ticks,Value,true);	//With 'true' set, strategy will manage stop internaly
				if (Mode=="Price")	SetStopLoss(Name+I,CalculationMode.Price,Value,true);	//Set to 'false' to have stops placed on brokers book				
			}	
		}//ENDsetMultiStop()
		//
		private bool tradingAllowed()  // Called if user selected a trade time limit and/or profit target for the session.
		{	if (Bars.FirstBarOfSession)	//If first bar of session, move previous session total to cumulative total bucket
			{	if(trace)	Print("FirstBarOfSession");
				priorTradesCumProfit = Performance.AllTrades.TradesPerformance.Currency.CumProfit;
			}	
			profitThisSession=Performance.AllTrades.TradesPerformance.Currency.CumProfit - priorTradesCumProfit;	//store session closed trade profit/losses
			if ((Position.GetProfitLoss(Close[0],PerformanceUnit.Currency)+profitThisSession)>profitTarget)			//if open trade profits plus closed session trade profits reach target add them together
			{	profitThisSession=profitThisSession+Position.GetProfitLoss(Close[0],PerformanceUnit.Currency);		//this will trigger trade violation and close trading for session
			}
			return ( (startTime1==0&&startTime2==0)||((ToTime(Time[0])>=startTime1*100&&ToTime(Time[0])<stopTime1*100)||(ToTime(Time[0])>=startTime2*100&&ToTime(Time[0])<stopTime2*100)&&(profitTarget==0||profitThisSession<=profitTarget)) );
		}//ENDprofitThisSession()
		//
		#region Expand/Colapse Control
        private void ModifyProperties(PropertyDescriptorCollection parm)
        {   
			if (autoTrend==AutoTrends.Disabled			) {	parm.Remove(parm.Find("EnterBreaks", true));	parm.Remove(parm.Find("Alert", true));			parm.Remove(parm.Find("ShowHistory", true));	parm.Remove(parm.Find("Strength", true));     		}
			if (rsi2==RSI2s.Disabled 					) {   parm.Remove(parm.Find("RSIPeriod", true));	parm.Remove(parm.Find("RSILimit", true));		parm.Remove(parm.Find("RSISmooth", true));		parm.Remove(parm.Find("TriggerHi", true));	parm.Remove(parm.Find("TriggerLow", true));		}
//			if (scalper!=Scalpers.Disabled				) { }
			if (macdCross==MacdCrosses.Disabled 		) {	parm.Remove(parm.Find("MacdSignal", true));		parm.Remove(parm.Find("MacdLimit", true));		parm.Remove(parm.Find("MacdMinDiff", true));	parm.Remove(parm.Find("MacdSlow", true));	parm.Remove(parm.Find("MacdFast", true));		parm.Remove(parm.Find("MacdSmooth", true));     }
			if (emaCross==EmaCrosses.Disabled			) {	parm.Remove(parm.Find("EmaLimit", true));		parm.Remove(parm.Find("EmaFast", true));		parm.Remove(parm.Find("EmaSlow", true));     }
			if (trendChannel==TrendChannels.Disabled	) {	parm.Remove(parm.Find("ChannelLimit", true));	parm.Remove(parm.Find("Dev", true));			parm.Remove(parm.Find("MALength", true));		parm.Remove(parm.Find("RSILength", true));	parm.Remove(parm.Find("XL", true));				parm.Remove(parm.Find("XS", true));				}
			if (smaSlope==SMASlopes.Disabled 			) {	parm.Remove(parm.Find("Indicate", true));		parm.Remove(parm.Find("TradeAngle", true)); 	parm.Remove(parm.Find("Limits", true)); 		parm.Remove(parm.Find("Period", true));		parm.Remove(parm.Find("Smooth", true));			parm.Remove(parm.Find("Lookback", true));		}
//			if (indicate!=Indicator.Slopes.Indicates.RSI) {	parm.Remove(parm.Find("Smooth", true));		}
			if (longTrend==LongTrends.Disabled			) {	parm.Remove(parm.Find("LongTrendPeriod",true));	parm.Remove(parm.Find("LongTrendAngle", true));     }
			if (trailStop==TrailStops.Disabled			) {	parm.Remove(parm.Find("StopTrailValue",true));	parm.Remove(parm.Find("StopATRMult",true));		parm.Remove(parm.Find("StopSMAPeriod",true));	}	
			if (trailStop==TrailStops.Fixed				) {													parm.Remove(parm.Find("StopATRMult",true));		parm.Remove(parm.Find("StopSMAPeriod",true));	}	
			if (trailStop==TrailStops.ATR				) {	parm.Remove(parm.Find("StopTrailValue",true));													parm.Remove(parm.Find("StopSMAPeriod",true));	}	
			if (trailStop==TrailStops.SMA				) {	parm.Remove(parm.Find("StopTrailValue",true));	parm.Remove(parm.Find("StopATRMult",true));														}	
		}
		#endregion
		//
	#endregion
		//
	#region Properties
		#region AutoTrendH Controls
			[Description("Allow Entries/Exits if price Above/Below rising/falling AutoTrendH lines")]
			[GridCategory("*AutoTrendH Control")][RefreshProperties(RefreshProperties.All)]	[Gui.Design.DisplayNameAttribute("*Enable Control")]		
			public AutoTrends AutoTrend		{   get { return autoTrend; 			}	set { autoTrend = value; 							}		}
			//
			[Description("Trigger Long entrys if price above any trendline, Short entrys if price below any trendline")]
			[GridCategory("*AutoTrendH Control")]
			public bool EnterBreaks			{   get { return enterBreaks; 			}	set { enterBreaks = value; 							}		}
			//
			[Description("Sets audible and logs alert if set to true")]
			[GridCategory("*AutoTrendH Control")]
			public bool Alert				{   get { return alert; 				}	set { alert = value; 								}		}
			//
			[Description("Show historical trendlines of auto-generated Trends")]
			[GridCategory("*AutoTrendH Control")]
			public bool ShowHistory			{   get { return showHistory; 			}	set { showHistory = value; 							}		}
			//
			[Description("Sets the granularity of trend detection (smaller # = shorter trend detection")]
			[GridCategory("*AutoTrendH Control")]
			public int Strength				{   get { return strength; 				}	set { strength=Math.Max(1,value); 					}		}
		#endregion
		//
		#region EMA_Cross
			[Description("Allow Entrys/Exits on Fast/Slow EMA crossings ")]
			[GridCategory("*EMA Crossings Control")][RefreshProperties(RefreshProperties.All)]	[Gui.Design.DisplayNameAttribute("*Enable Control")]		
			public EmaCrosses EmaCross		{   get { return emaCross; 				}	set { emaCross = value; 							}		}
			//
			[Description("Allow Entrys/Exits if price Above/Below EMA Slow plot ")]
			[GridCategory("*EMA Crossings Control")][RefreshProperties(RefreshProperties.All)]	[Gui.Design.DisplayNameAttribute("*Enable Limits")]		
			public EmaLimits EmaLimit		{   get { return emaLimit; 				}	set { emaLimit = value; 							}		}
			//
			[Description("Fast EMA Period Signal Line")]
			[GridCategory("*EMA Crossings Control")]		
			public int EmaFast				{   get { return emaFast;				}	set { emaFast = value; 							}		}
			//
			[Description("Slow EMA Period Signal line")]
			[GridCategory("*EMA Crossings Control")]
			public int EmaSlow				{   get { return emaSlow; 				}	set { emaSlow = value; 							}		}
			//
		#endregion
		//
		#region MACD_Cross
			[Description("Allow Entrys/Exits on MACD cross Above/below Signal line")]
			[GridCategory("*MACD Crossover Control")][RefreshProperties(RefreshProperties.All)]	[Gui.Design.DisplayNameAttribute("* Enable Control")]		
			public MacdCrosses MacdCross	{   get { return macdCross; 			}	set { macdCross = value; 							}		}
			//
			[Description("Generate Signals from Fast/Slow Crosses or Rising/Falling Diff's")]
			[GridCategory("*MACD Crossover Control")]	[Gui.Design.DisplayNameAttribute("* Signal Triggers")]		
			public MacdSignals MacdSignal	{   get { return macdSignal; 			}	set { macdSignal = value; 							}		}
			//
			[Description("Limit trades to direction of MACD.Avg plot")]
			[GridCategory("*MACD Crossover Control")]	[Gui.Design.DisplayNameAttribute("MACD Limit")]		
			public bool MacdLimit			{   get { return macdLimit;				}	set { macdLimit = value; 							}		}
			//
			[Description("Minimum Difference in MA's (pips/ticks) needed to allow entries (reduce congestion trading) 0=Disabled")]
			[GridCategory("*MACD Crossover Control")]	[Gui.Design.DisplayNameAttribute("*Min. Diff. Limit")]		
			public int MacdMinDiff			{   get { return macdMinDiff; 			}	set { macdMinDiff = value; 							}		}
			//
			[Description("MovingAverage period for Fast MACD line")]
			[GridCategory("*MACD Crossover Control")]		
			public int MacdFast				{   get { return macdFast;				}	set { macdFast = value; 							}		}
			//
			[Description("Moving Average period for Slow Signal line")]
			[GridCategory("*MACD Crossover Control")]
			public int MacdSlow				{   get { return macdSlow; 				}	set { macdSlow = value; 							}		}
			//
			[Description("MACD Smooth period")]
			[GridCategory("*MACD Crossover Control")]
			public int MacdSmooth			{   get { return macdSmooth; 			}	set { macdSmooth = value; 							}		}
			//
		#endregion
		//
		#region RSI Controls
			[Description("Allow Entries/Exits if price Above/Below RSIUpper/RSILower ")]
			[GridCategory("*RSI Control")][RefreshProperties(RefreshProperties.All)][Gui.Design.DisplayNameAttribute("*Enable Control")]		
			public RSI2s Rsi2				{   get { return rsi2;					}	set { rsi2			= value; 			}		}
			//
			[Description("Only allow positions Above/Below thresholds")]
			[GridCategory("*RSI Control")]	[Gui.Design.DisplayNameAttribute("RSI Limit")]
			public bool RSILimit			{   get { return rsi2Limit;				}	set { rsi2Limit	= value; 				}		}
			//
			[Description("Lookback Period of RSI indicator")]
			[GridCategory("*RSI Control")]
			public int RSIPeriod			{   get { return rsi2Period;			}	set { rsi2Period	= value; 			}		}
			//
			[Description("Smoothing Period of RSI")]
			[GridCategory("*RSI Control")]
			public int RSISmooth			{   get { return rsi2Smooth;			}	set { rsi2Smooth	= value; 			}		}
			//
			[Description("Upper RSI trigger threshold")]
			[GridCategory("*RSI Control")]
			public double TriggerHi			{   get { return rsi2Upper; 			}	set { rsi2Upper 	= value; 			}		}
			//
			[Description("Lower RSI trigger threshold")]
			[GridCategory("*RSI Control")]
			public double TriggerLow		{   get { return rsi2Lower; 			}	set { rsi2Lower		= value; 			}		}
			//
		#endregion		
		//
		#region Scalper Controls
			[Description("Trigger Entries/Exits on Scalper indicator arrows")]
			[GridCategory("*Scalper Control")][RefreshProperties(RefreshProperties.All)]	[Gui.Design.DisplayNameAttribute("*Enable Control")]		
			public Scalpers Scalp		{   get { return scalper; 			}	set { scalper = value; 										}		}
		#endregion
		//
		#region TrendChannel Controls
			[Description("Allow Entries/Exits if price is trending (red or green trending dots on trend channel)")]
			[GridCategory("*TrendChannel Control")][RefreshProperties(RefreshProperties.All)]	[Gui.Design.DisplayNameAttribute("* Enable Control")]		
			public TrendChannels Trend_Channel	{   get { return trendChannel; 		}	set { trendChannel = value; 						}		}
			//
			[Description("Only allow Entries if price is outside of TrendChannel (Channel Breakouts) Enabling Exits will exit if open position moves beyond middle of channel. NOTE: Use Deviation to set channel width")]
			[GridCategory("*TrendChannel Control")]		[Gui.Design.DisplayNameAttribute("*ChannelLimit")]		
			public ChannelLimits ChannelLimit	{   get { return channelLimit; 		}	set { channelLimit = value; 						}		}
			//
			[Description("Standard Deviation Trend Channel Width (for use with channel limit)")]
			[GridCategory("*TrendChannel Control")]		[Gui.Design.DisplayNameAttribute("Deviation")]
			public double Dev					{   get { return dev; 				}	set { dev=Math.Max(0.00001,value);					}		}
			//
			[Description("Moving Average Lookback Period (Affects sensitivity of upper/lower channal lines)")]
			[GridCategory("*TrendChannel Control")]		[Gui.Design.DisplayNameAttribute("MA Period")]
			public int MALength					{   get { return maLength; 			}	set { maLength=Math.Max(1,value); 					}		}
			//
			[Description("RSI Lookback Period (Affects sensitivity of the dots)")]
			[GridCategory("*TrendChannel Control")]		[Gui.Design.DisplayNameAttribute("RSI Period")]
			public int RSILength				{   get { return rsiLength; 		}	set { rsiLength=Math.Max(1,value);					}		}
			//
			[Description(":Min MA angle (>50) to allow Long entry (higher value=harder to enter trade)")]
			[GridCategory("*TrendChannel Control")]		[Gui.Design.DisplayNameAttribute("EntryLevLg")]
			public double XL					{   get { return xl;	 			}	set { xl = Math.Max(50, value);						}		}
			//
			[Description(":Max MA angle (<50) to allow Short entry (lower value=harder to enter trade)")]
			[GridCategory("*TrendChannel Control")]		[Gui.Design.DisplayNameAttribute("EntryLevSh")]
			public double XS					{   get { return xs; 				}	set { xs = Math.Min(50, value); 					}		}
			//
		#endregion 
		//
		#region Slopes Controls
			[Description("Allow Entrys/Exits on Rising/Falling color slope")]
			[GridCategory("*Slope Control")][RefreshProperties(RefreshProperties.All)]	[Gui.Design.DisplayNameAttribute("*Enable Control")]		
			public SMASlopes SMASlope			{   get { return smaSlope; 			}	set { smaSlope = value; 							}		}
			//
			[Description("Select Indicator Type to for Slope calculations to control Entrys/Exits")]
			[GridCategory("*Slope Control")][RefreshProperties(RefreshProperties.All)]	[Gui.Design.DisplayNameAttribute("*Indicator Type")]		
			public Slopes.Indicates Indicate	{   get { return indicate;			}	set { indicate = value;								}		}
			//
			[Description("Minimum (+/- 0-90 degrees) indicator slope Angle to allow trades (no signal generated during yellow portion of slope)")]
			[GridCategory("*Slope Control")]
			public int TradeAngle				{   get { return tradeAngle; 		}	set { tradeAngle = value; 							}		}
			//
			[Description("Limits trades Longs above Slopes indicator / Shorts below Slopes Trendline")]
			[GridCategory("*Slope Control")]
			public bool Limits					{   get { return smaLimits;			}	set { smaLimits = value; 							}		}
			//
			[Description("*Number of bars back to base slope calculations on (Typically 1-6)")]
			[GridCategory("*Slope Control")]
			public int Lookback					{   get { return smaLookback; 		}	set { smaLookback = value;							}		}
			//
			[Description("Indicator period")]
			[GridCategory("*Slope Control")]
			public int Period					{   get { return smaPeriod; 		}	set { smaPeriod = value;							}		}
			//
			[Description("RSI Smoothing period")]
			[GridCategory("*Slope Control")]
			public int Smooth					{   get { return smaSmooth;			}	set { smaSmooth = value;							}		}
			//
		#endregion
		//
		#region EntryLimits
			[Description("NOTE: Control is only for Entrys;  Allow Entries only when price Above/Below LongTerm SMA Trend line, only on Rising/Falling long trend line or both Above/Below while trend line is Rising/Falling")]
			[GridCategory("Entry Limits")][RefreshProperties(RefreshProperties.All)]		[Gui.Design.DisplayNameAttribute("*Enable LongTrend Control")]		
			public LongTrends LongTrend				{   get { return longTrend; 		}	set { longTrend = value; 							}		}
			//
			[Description("Allows entries only in direction of longterm SMA trend (Only allow Longs if rising, Shorts if falling) Recommend 60-120 : 0=Disabled) NOTE: Use LongTrendAngle to adjust")]
			[GridCategory("Entry Limits")]		[Gui.Design.DisplayNameAttribute("LongTrend Period")]		
			public int 		LongTrendPeriod			{   get { return longTrendPeriod; 	}	set { longTrendPeriod = value;						}		}
			//
			[Description("Minimum Long Trend SMA +/- Angle(0-90) to allow entrys")]
			[GridCategory("Entry Limits")]
			public int 		LongTrendAngle			{   get { return longTrendAngle; 	}	set { longTrendAngle=Math.Min(value,90);}}
			//
			[Description("Allow entries Long/Short/Both")]
			[GridCategory("Entry Limits")]		[Gui.Design.DisplayNameAttribute("Entrys Limit")]
			public EntrysLimits	EntrysLimit			{   get { return entrysLimit; 		}	set { entrysLimit = value; 							}		}
			//
			[Description("Reverse Entry/Exit signals (if you get truly horrible backtest results then try and reverse triggers :-)")]
			[GridCategory("Entry Limits")]
			public bool		ReverseTriggers			{   get { return atOppisite; 		}	set { atOppisite = value; 							}		}
			//
			[Description("Only allow entry if previous bar is in direction of entry trigger (Good for preventing immediate re-entry in wrong direction if position exited on a Stop)")]
			[GridCategory("Entry Limits")]		[Gui.Design.DisplayNameAttribute("Previous Bar Limit")]		
			public bool 	PreviousBarLimit		{   get { return prevBarLimit; 		}	set { prevBarLimit = value;							}		}
			//
			[Description("Dont allow re-entry in same direction until Neutral or Reverse trigger signal encountered")]
			[GridCategory("Entry Limits")]		[Gui.Design.DisplayNameAttribute("ReEntry Prevention")]	
			public bool		Prevent					{   get { return prevent; 			}	set { prevent = value; 								}		}
			//
			[Description("Prevent re-entrys for X bars after exits (prevents immediate re-entry if stopped out and indicators are still indicating entry signal) 0=no delay")]
			[GridCategory("Entry Limits")]		[Gui.Design.DisplayNameAttribute("ReEntry Bars Delay")]		
			public int		EntryDelay				{   get { return entryDelay; 		}	set { entryDelay	= value;						}		}
			//
		#endregion
		//
		#region Trade Management
			[Description("Select/Enter ATM to manage trade after Stratgy entry(Allows adding size/moving stops/SuperDom/ChartTrader control) Ignores Stops/MaxEntrys listed below. Leave blank or 0 to disable NOTE:Only operative on live or market replay")]
			[GridCategory("Manage Trade")]	[TypeConverter(typeof(AtmConverter))]	[Gui.Design.DisplayName(" ATM handoff template")]
			public string	ATMTemplate				{	get { return _ATMTemplate; 		}	set {_ATMTemplate=value;							}		}
			//
			[Description("Select TrailStop Method...  1)Fixed #pips/ticks behind entry  2)% of ATR(30) 'AverageTradingRange'   3)SMA (Exit when close above/below SMA    !NOTE!: A blue dot will dynamicaly move above/below entry bar showing position of TrailStop")]
			[GridCategory("Manage Trade")][RefreshProperties(RefreshProperties.All)]	[Gui.Design.DisplayNameAttribute("Trail Stop Method")]		
			public TrailStops TrailStop				{   get { return trailStop; 		}	set { trailStop	= value;							}		}
			//
			[Description("TrailMethod...'Fixed' (enter #pips to trail price Ex.10-100) ")]
			[GridCategory("Manage Trade")]		[Gui.Design.DisplayNameAttribute("TrailStop Value")]		
			public int		StopTrailValue			{   get { return stopTrailValue;	}	set { stopTrailValue=value; 						}		}
			//
			[Description("TrailMethod...'SMA' (enter SMA period Ex.5-25")]
			[GridCategory("Manage Trade")]		[Gui.Design.DisplayNameAttribute("TrailSMA Period")]		
			public int		StopSMAPeriod			{   get { return stopTrailPeriod;	}	set { stopTrailPeriod=value; 						}		}
			//
			[Description("TrailMethod...'ATR' (enter Multiplier of ATR(30) Ex. 2.5")]
			[GridCategory("Manage Trade")]		[Gui.Design.DisplayNameAttribute("TrailATR Mult.")]		
			public double	StopATRMult				{   get { return stopTrailMult;		}	set { stopTrailMult=value; 							}		}
			//
			[Description("Set Verbose mode on for output to Log window")]
			[GridCategory("Manage Trade")]
			public bool		Trace					{   get { return trace; 			}	set { trace = value;								}		}  
			//
			[Description("Turns on stop setting chart markers (colored dots) ")]
			[GridCategory("Manage Trade")]
			public bool		StopsMarkers			{   get { return stopMarkers; 		}	set { stopMarkers=value;							}		}  
			//
			[Description("Maximum simultaneous entries allowed (Note: Additional Entries on first bars close more profitable than previous entry(s) )")]
			[GridCategory("Manage Trade")]
			public int		MaxEntries				{   get { return maxEntries; 		}	set { maxEntries=Math.Max(0,value);					}		}  
			//
			[Description("Lock in profits after StopProfit pips/ticks (set stopTrailValue value for distance behind StopProfit) 0=disabled")]
			[GridCategory("Manage Trade")]
			public int		StopProfit				{   get { return stopProfit; 		}	set { stopProfit=value;		 						}		}
			//
			[Description("Exits position(s) if adverse move of StopShock pips/ticks occur with 1 minute granularity (Caution: Only tests for move each minute so 300 pip move in 10 seconds will not trip till minute is up)")]
			[GridCategory("Manage Trade")]
			public int		StopShock				{   get { return stopShock; 		}	set { stopShock = value; 							}		}
			//
			[Description("Initial StopLoss in Pips/Ticks setting on entry (0=disabled)")]
			[GridCategory("Manage Trade")]
			public int		StopLoss				{   get { return stopLoss; 			}	set { stopLoss = value; 							}		}
			//
			[Description("Move stop to breakeven (entry) after StopEven pips/ticks profitable (0=disabled) A Yellow dot will appear on entry bar to signal when BreakEven stop is activated")]
			[GridCategory("Manage Trade")]
			public int		StopEven				{   get { return stopEven; 			}	set { stopEven = value;								}		}
			//
		#endregion
		//
		#region Tradeing Times
			[Description("Allows trading only after StartTime (Note:0900 = 9:00AM; 0=This Trading Time section disabled)ATENTION: Must have your SessionTemplate active so that ProfitTarget will reset on SessionChange!")]
			[GridCategory("Trading Time Control")][Gui.Design.DisplayNameAttribute("1)Start Time")]
			public int StartTime1				{   get { return startTime1; 		}	set { startTime1 = value; 							}		}
			//
			[Description("Allows trading only before StopTime (Note:1600= 4:00PM) A diamond mark will show on the bars that trading is not allowed.")]
			[GridCategory("Trading Time Control")][Gui.Design.DisplayNameAttribute("1)Stop Time")]
			public int StopTime1				{   get { return stopTime1;		 	}	set { stopTime1 = value; 							}		}
			//
			[Description("Allows trading only after StartTime (Note:0900 = 9:00AM; 0=This Trading Time section disabled)ATENTION: Must have your SessionTemplate active so that ProfitTarget will reset on SessionChange!")]
			[GridCategory("Trading Time Control")][Gui.Design.DisplayNameAttribute("2)Start Time")]
			public int StartTime2				{   get { return startTime2; 		}	set { startTime2 = value; 							}		}
			//
			[Description("Allows trading only before StopTime (Note:1600= 4:00PM) A diamond mark will show on the bars that trading is not allowed.")]
			[GridCategory("Trading Time Control")][Gui.Design.DisplayNameAttribute("2)Stop Time")]
			public int StopTime2				{   get { return stopTime2;		 	}	set { stopTime2 = value; 							}		}
			//
			[Description("Allows trading only until ProfitTarget reached (Enter Dollar Amount)")]
			[GridCategory("Trading Time Control")]
			public double ProfitTarget			{   get { return profitTarget; 		}	set { profitTarget = value;						 	}		}
			//
		#endregion
		//
    #region ICustomTypeDescriptor Members
		public AttributeCollection 			GetAttributes()							{	return TypeDescriptor.GetAttributes(GetType());				}
		public string 						GetClassName()							{	return TypeDescriptor.GetClassName(GetType());				}
		public string 						GetComponentName()						{	return TypeDescriptor.GetComponentName(GetType());			}
		public TypeConverter 				GetConverter()							{	return TypeDescriptor.GetConverter(GetType());				}
		public EventDescriptor 				GetDefaultEvent()						{	return TypeDescriptor.GetDefaultEvent(GetType());			}
		public PropertyDescriptor 			GetDefaultProperty()					{	return TypeDescriptor.GetDefaultProperty(GetType());        }
		public object 						GetEditor(Type editorBaseType)			{	return TypeDescriptor.GetEditor(GetType(), editorBaseType);	}
		public EventDescriptorCollection 	GetEvents(Attribute[] attributes)		{	return TypeDescriptor.GetEvents(GetType(), attributes);		}
		public EventDescriptorCollection 	GetEvents()								{	return TypeDescriptor.GetEvents(GetType());					}
		public PropertyDescriptorCollection GetProperties()							{	return TypeDescriptor.GetProperties(GetType());				}
		public object 						GetPropertyOwner(PropertyDescriptor pd)	{	return this;												}
		public PropertyDescriptorCollection GetProperties(Attribute[] attributes)
		{   PropertyDescriptorCollection 	orig = TypeDescriptor.GetProperties(GetType(), attributes);
			PropertyDescriptor[] 			arr = new PropertyDescriptor[orig.Count];
			orig.CopyTo(arr, 0);
			PropertyDescriptorCollection 	col = new PropertyDescriptorCollection(arr);
			ModifyProperties(col);
			return col;        
        }


    #endregion
			
	#endregion
    }
}
